@@ -4,10 +4,6 @@
 
 package ssa
 
-import (
-	"fmt"
-)
-
 // ----------------------------------------------------------------------------
 // Sparse Conditional Constant Propagation
 //
@@ -118,7 +114,7 @@ func sccp(f *Func) {
 	constCnt, rewireCnt := t.replaceConst()
 	if f.pass.debug > 0 {
 		if constCnt > 0 || rewireCnt > 0 {
-			fmt.Printf("Phase SCCP for %v : %v constants, %v dce\n", f.Name, constCnt, rewireCnt)
+			f.Warnl(f.Entry.Pos, "Phase SCCP for %v : %v constants, %v dce", f.Name, constCnt, rewireCnt)
 		}
 	}
 }
@@ -377,7 +373,7 @@ func (t *worklist) visitValue(val *Value) {
 		// re-visit all uses of value if its lattice is changed
 		newLt := t.getLatticeCell(val)
 		if !equals(newLt, oldLt) {
-			if int8(oldLt.tag) > int8(newLt.tag) {
+			if oldLt.tag > newLt.tag {
 				t.f.Fatalf("Must lower lattice\n")
 			}
 			t.addUses(val)
@@ -567,7 +563,7 @@ func (t *worklist) replaceConst() (int, int) {
 		if lt.tag == constant {
 			if !isConst(val) {
 				if t.f.pass.debug > 0 {
-					fmt.Printf("Replace %v with %v\n", val.LongString(), lt.val.LongString())
+					t.f.Warnl(val.Pos, "Replace %v with %v", val.LongString(), lt.val.LongString())
 				}
 				val.reset(lt.val.Op)
 				val.AuxInt = lt.val.AuxInt
@@ -579,7 +575,7 @@ func (t *worklist) replaceConst() (int, int) {
 				if rewireSuccessor(block, lt.val) {
 					rewireCnt++
 					if t.f.pass.debug > 0 {
-						fmt.Printf("Rewire %v %v successors\n", block.Kind, block)
+						t.f.Warnl(block.Pos, "Rewire %v %v successors", block.Kind, block)
 					}
 				}
 			}
