@@ -223,7 +223,7 @@ func testArithConstShift(t *testing.T) {
 	}
 }
 
-// overflowConstShift_ssa verifies that constant folding for shift
+// overflowConstShift64_ssa verifies that constant folding for shift
 // doesn't wrap (i.e. x << MAX_INT << 1 doesn't get folded to x << 0).
 //
 //go:noinline
@@ -444,6 +444,19 @@ func testBitwiseRshU_ssa(a uint32, b, c uint32) uint32 {
 	return a >> b >> c
 }
 
+//go:noinline
+func orLt_ssa(x int) bool {
+	y := x - x
+	return (x | 2) < y
+}
+
+// test riscv64 SLTI rules
+func testSetIfLessThan(t *testing.T) {
+	if want, got := true, orLt_ssa(-7); got != want {
+		t.Errorf("orLt_ssa(-7) = %t want %t", got, want)
+	}
+}
+
 //go:noinline
 func testShiftCX_ssa() int {
 	v1 := uint8(3)
@@ -977,6 +990,7 @@ func TestArithmetic(t *testing.T) {
 	testRegallocCVSpill(t)
 	testSubqToNegq(t)
 	testBitwiseLogic(t)
+	testSetIfLessThan(t)
 	testOcom(t)
 	testLrot(t)
 	testShiftCX(t)
@@ -1390,11 +1404,17 @@ func div19_int64(n int64) bool {
 	return n%19 == 0
 }
 
+var (
+	// These have to be global to avoid getting constant-folded in the function body:
+	// as locals, prove can see that they are actually constants.
+	sixU, nineteenU uint64 = 6, 19
+	sixS, nineteenS int64  = 6, 19
+)
+
 // testDivisibility confirms that rewrite rules x%c ==0 for c constant are correct.
 func testDivisibility(t *testing.T) {
 	// unsigned tests
 	// test an even and an odd divisor
-	var sixU, nineteenU uint64 = 6, 19
 	// test all inputs for uint8, uint16
 	for i := uint64(0); i <= math.MaxUint16; i++ {
 		if i <= math.MaxUint8 {
@@ -1402,7 +1422,7 @@ func testDivisibility(t *testing.T) {
 				t.Errorf("div6_uint8(%d) = %v want %v", i, got, want)
 			}
 			if want, got := uint8(i)%uint8(nineteenU) == 0, div19_uint8(uint8(i)); got != want {
-				t.Errorf("div6_uint19(%d) = %v want %v", i, got, want)
+				t.Errorf("div19_uint8(%d) = %v want %v", i, got, want)
 			}
 		}
 		if want, got := uint16(i)%uint16(sixU) == 0, div6_uint16(uint16(i)); got != want {
@@ -1450,7 +1470,6 @@ func testDivisibility(t *testing.T) {
 
 	// signed tests
 	// test an even and an odd divisor
-	var sixS, nineteenS int64 = 6, 19
 	// test all inputs for int8, int16
 	for i := int64(math.MinInt16); i <= math.MaxInt16; i++ {
 		if math.MinInt8 <= i && i <= math.MaxInt8 {
@@ -1458,7 +1477,7 @@ func testDivisibility(t *testing.T) {
 				t.Errorf("div6_int8(%d) = %v want %v", i, got, want)
 			}
 			if want, got := int8(i)%int8(nineteenS) == 0, div19_int8(int8(i)); got != want {
-				t.Errorf("div6_int19(%d) = %v want %v", i, got, want)
+				t.Errorf("div19_int8(%d) = %v want %v", i, got, want)
 			}
 		}
 		if want, got := int16(i)%int16(sixS) == 0, div6_int16(int16(i)); got != want {
