@@ -103,12 +103,15 @@ func ClosureType(clo *ir.ClosureExpr) *types.Type {
 
 	fields := make([]*types.Field, 1+len(clo.Func.ClosureVars))
 	fields[0] = types.NewField(base.AutogeneratedPos, types.LocalPkg.Lookup("F"), types.Types[types.TUINTPTR])
-	for i, v := range clo.Func.ClosureVars {
-		typ := v.Type()
-		if !v.Byval() {
-			typ = types.NewPtr(typ)
+	it := NewClosureStructIter(clo.Func.ClosureVars)
+	i := 0
+	for {
+		n, typ, _ := it.Next()
+		if n == nil {
+			break
 		}
 		fields[1+i] = types.NewField(base.AutogeneratedPos, types.LocalPkg.LookupNum("X", i), typ)
+		i++
 	}
 	typ := types.NewStruct(fields)
 	typ.SetNoalg(true)
@@ -755,17 +758,7 @@ func tcRecover(n *ir.CallExpr) ir.Node {
 		return n
 	}
 
-	// FP is equal to caller's SP plus FixedFrameSize.
-	var fp ir.Node = ir.NewCallExpr(n.Pos(), ir.OGETCALLERSP, nil, nil)
-	if off := base.Ctxt.Arch.FixedFrameSize; off != 0 {
-		fp = ir.NewBinaryExpr(n.Pos(), ir.OADD, fp, ir.NewInt(base.Pos, off))
-	}
-	// TODO(mdempsky): Replace *int32 with unsafe.Pointer, without upsetting checkptr.
-	fp = ir.NewConvExpr(n.Pos(), ir.OCONVNOP, types.NewPtr(types.Types[types.TINT32]), fp)
-
-	n.SetOp(ir.ORECOVERFP)
 	n.SetType(types.Types[types.TINTER])
-	n.Args = []ir.Node{Expr(fp)}
 	return n
 }
 
@@ -832,3 +825,37 @@ func tcUnsafeString(n *ir.BinaryExpr) *ir.BinaryExpr {
 	n.SetType(types.Types[types.TSTRING])
 	return n
 }
+
+// ClosureStructIter iterates through a slice of closure variables returning
+// their type and offset in the closure struct.
+type ClosureStructIter struct {
+	closureVars []*ir.Name
+	offset      int64
+	next        int
+}
+
+// NewClosureStructIter creates a new ClosureStructIter for closureVars.
+func NewClosureStructIter(closureVars []*ir.Name) *ClosureStructIter {
+	return &ClosureStructIter{
+		closureVars: closureVars,
+		offset:      int64(types.PtrSize), // PtrSize to skip past function entry PC field
+		next:        0,
+	}
+}
+
+// Next returns the next name, type and offset of the next closure variable.
+// A nil name is returned after the last closure variable.
+func (iter *ClosureStructIter) Next() (n *ir.Name, typ *types.Type, offset int64) {
+	if iter.next >= len(iter.closureVars) {
+		return nil, nil, 0
+	}
+	n = iter.closureVars[iter.next]
+	typ = n.Type()
+	if !n.Byval() {
+		typ = types.NewPtr(typ)
+	}
+	iter.next++
+	offset = types.RoundUp(iter.offset, typ.Alignment())
+	iter.offset = offset + typ.Size()
+	return n, typ, offset
+}
