@@ -99,8 +99,7 @@ var testObjects = []struct {
 	{"type t = struct{f int}", "t", "type p.t = struct{f int}", false},
 	{"type t = func(int)", "t", "type p.t = func(int)", false},
 	{"type A = B; type B = int", "A", "type p.A = p.B", true},
-	{"type A[P ~int] = struct{}", "A", "type p.A[P ~int] = struct{}", true}, // requires GOEXPERIMENT=aliastypeparams
-
+	{"type A[P ~int] = struct{}", "A", "type p.A[P ~int] = struct{}", true},
 	{"var v int", "v", "var p.v int", false},
 
 	{"func f(int) string", "f", "func p.f(int) string", false},
@@ -114,10 +113,6 @@ func TestObjectString(t *testing.T) {
 
 	for i, test := range testObjects {
 		t.Run(fmt.Sprint(i), func(t *testing.T) {
-			if test.alias {
-				revert := setGOEXPERIMENT("aliastypeparams")
-				defer revert()
-			}
 			src := "package p; " + test.src
 			conf := Config{Error: func(error) {}, Importer: defaultImporter(), EnableAlias: test.alias}
 			pkg, err := typecheck(src, &conf, nil)
