@@ -6,10 +6,11 @@ package types2
 
 import (
 	"cmd/compile/internal/syntax"
+	"cmp"
 	"fmt"
 	"go/constant"
 	. "internal/types/errors"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"unicode"
@@ -18,6 +19,7 @@ import (
 // A declInfo describes a package-level const, type, var, or func declaration.
 type declInfo struct {
 	file      *Scope           // scope of file containing this declaration
+	version   goVersion        // Go version of file containing this declaration
 	lhs       []*Var           // lhs of n:1 variable declarations, or nil
 	vtyp      syntax.Expr      // type, or nil (for const and var declarations only)
 	init      syntax.Expr      // init/orig expression, or nil (for const and var declarations only)
@@ -167,10 +169,7 @@ func (check *Checker) importPackage(pos syntax.Pos, path, dir string) *Package {
 			if imp == nil {
 				// create a new fake package
 				// come up with a sensible package name (heuristic)
-				name := path
-				if i := len(name); i > 0 && name[i-1] == '/' {
-					name = name[:i-1]
-				}
+				name := strings.TrimSuffix(path, "/")
 				if i := strings.LastIndex(name, "/"); i >= 0 {
 					name = name[i+1:]
 				}
@@ -220,14 +219,17 @@ func (check *Checker) collectObjects() {
 		recv *syntax.Name // receiver type name
 	}
 	var methods []methodInfo // collected methods with valid receivers and non-blank _ names
-	var fileScopes []*Scope
+
+	fileScopes := make([]*Scope, len(check.files)) // fileScopes[i] corresponds to check.files[i]
 	for fileNo, file := range check.files {
+		check.version = asGoVersion(check.versions[file.Pos().FileBase()])
+
 		// The package identifier denotes the current package,
 		// but there is no corresponding package object.
 		check.recordDef(file.PkgName, nil)
 
 		fileScope := NewScope(pkg.scope, syntax.StartPos(file), syntax.EndPos(file), check.filename(fileNo))
-		fileScopes = append(fileScopes, fileScope)
+		fileScopes[fileNo] = fileScope
 		check.recordScope(file, fileScope)
 
 		// determine file directory, necessary to resolve imports
@@ -293,7 +295,7 @@ func (check *Checker) collectObjects() {
 
 				if imp.fake {
 					// match 1.17 cmd/compile (not prescribed by spec)
-					pkgName.used = true
+					check.usedPkgNames[pkgName] = true
 				}
 
 				// add import to file scope
@@ -362,7 +364,7 @@ func (check *Checker) collectObjects() {
 						init = values[i]
 					}
 
-					d := &declInfo{file: fileScope, vtyp: last.Type, init: init, inherited: inherited}
+					d := &declInfo{file: fileScope, version: check.version, vtyp: last.Type, init: init, inherited: inherited}
 					check.declarePkgObj(name, obj, d)
 				}
 
@@ -380,13 +382,13 @@ func (check *Checker) collectObjects() {
 					// The lhs elements are only set up after the for loop below,
 					// but that's ok because declarePkgObj only collects the declInfo
 					// for a later phase.
-					d1 = &declInfo{file: fileScope, lhs: lhs, vtyp: s.Type, init: s.Values}
+					d1 = &declInfo{file: fileScope, version: check.version, lhs: lhs, vtyp: s.Type, init: s.Values}
 				}
 
 				// declare all variables
 				values := syntax.UnpackListExpr(s.Values)
 				for i, name := range s.NameList {
-					obj := NewVar(name.Pos(), pkg, name.Value, nil)
+					obj := newVar(PackageVar, name.Pos(), pkg, name.Value, nil)
 					lhs[i] = obj
 
 					d := d1
@@ -396,7 +398,7 @@ func (check *Checker) collectObjects() {
 						if i < len(values) {
 							init = values[i]
 						}
-						d = &declInfo{file: fileScope, vtyp: s.Type, init: init}
+						d = &declInfo{file: fileScope, version: check.version, vtyp: s.Type, init: init}
 					}
 
 					check.declarePkgObj(name, obj, d)
@@ -409,7 +411,7 @@ func (check *Checker) collectObjects() {
 
 			case *syntax.TypeDecl:
 				obj := NewTypeName(s.Name.Pos(), pkg, s.Name.Value, nil)
-				check.declarePkgObj(s.Name, obj, &declInfo{file: fileScope, tdecl: s})
+				check.declarePkgObj(s.Name, obj, &declInfo{file: fileScope, version: check.version, tdecl: s})
 
 			case *syntax.FuncDecl:
 				name := s.Name.Value
@@ -434,10 +436,8 @@ func (check *Checker) collectObjects() {
 					if name == "init" {
 						obj.parent = pkg.scope
 						check.recordDef(s.Name, obj)
-						// init functions must have a body
 						if s.Body == nil {
-							// TODO(gri) make this error message consistent with the others above
-							check.softErrorf(obj.pos, MissingInitBody, "missing function body")
+							check.softErrorf(obj.pos, MissingInitBody, "func init must have a body")
 						}
 					} else {
 						check.declare(pkg.scope, s.Name, obj, nopos)
@@ -445,17 +445,17 @@ func (check *Checker) collectObjects() {
 				} else {
 					// method
 					// d.Recv != nil
-					ptr, recv, _ := check.unpackRecv(s.Recv.Type, false)
+					ptr, base, _ := check.unpackRecv(s.Recv.Type, false)
 					// Methods with invalid receiver cannot be associated to a type, and
 					// methods with blank _ names are never found; no need to collect any
 					// of them. They will still be type-checked with all the other functions.
-					if recv != nil && name != "_" {
+					if recv, _ := base.(*syntax.Name); recv != nil && name != "_" {
 						methods = append(methods, methodInfo{obj, ptr, recv})
 					}
 					check.recordDef(s.Name, obj)
 				}
 				_ = len(s.TParamList) != 0 && !hasTParamError && check.verifyVersionf(s.TParamList[0], go1_18, "type parameter")
-				info := &declInfo{file: fileScope, fdecl: s}
+				info := &declInfo{file: fileScope, version: check.version, fdecl: s}
 				// Methods are not package-level objects but we still track them in the
 				// object map so that we can handle them like regular functions (if the
 				// receiver is invalid); also we need their fdecl info when associating
@@ -492,51 +492,55 @@ func (check *Checker) collectObjects() {
 	// associate methods with receiver base type name where possible.
 	// Ignore methods that have an invalid receiver. They will be
 	// type-checked later, with regular functions.
-	if methods != nil {
-		check.methods = make(map[*TypeName][]*Func)
-		for i := range methods {
-			m := &methods[i]
-			// Determine the receiver base type and associate m with it.
-			ptr, base := check.resolveBaseTypeName(m.ptr, m.recv, fileScopes)
-			if base != nil {
-				m.obj.hasPtrRecv_ = ptr
-				check.methods[base] = append(check.methods[base], m.obj)
-			}
+	if methods == nil {
+		return
+	}
+
+	check.methods = make(map[*TypeName][]*Func)
+	for i := range methods {
+		m := &methods[i]
+		// Determine the receiver base type and associate m with it.
+		ptr, base := check.resolveBaseTypeName(m.ptr, m.recv)
+		if base != nil {
+			m.obj.hasPtrRecv_ = ptr
+			check.methods[base] = append(check.methods[base], m.obj)
 		}
 	}
 }
 
-// unpackRecv unpacks a receiver type and returns its components: ptr indicates whether
-// rtyp is a pointer receiver, rname is the receiver type name, and tparams are its
-// type parameters, if any. The type parameters are only unpacked if unpackParams is
-// set. If rname is nil, the receiver is unusable (i.e., the source has a bug which we
-// cannot easily work around).
-func (check *Checker) unpackRecv(rtyp syntax.Expr, unpackParams bool) (ptr bool, rname *syntax.Name, tparams []*syntax.Name) {
-L: // unpack receiver type
-	// This accepts invalid receivers such as ***T and does not
-	// work for other invalid receivers, but we don't care. The
-	// validity of receiver expressions is checked elsewhere.
-	for {
-		switch t := rtyp.(type) {
-		case *syntax.ParenExpr:
-			rtyp = t.X
-		// case *ast.StarExpr:
-		//      ptr = true
-		// 	rtyp = t.X
-		case *syntax.Operation:
-			if t.Op != syntax.Mul || t.Y != nil {
-				break
-			}
-			ptr = true
-			rtyp = t.X
-		default:
-			break L
-		}
+// sortObjects sorts package-level objects by source-order for reproducible processing
+func (check *Checker) sortObjects() {
+	check.objList = make([]Object, len(check.objMap))
+	i := 0
+	for obj := range check.objMap {
+		check.objList[i] = obj
+		i++
+	}
+	slices.SortFunc(check.objList, func(a, b Object) int {
+		return cmp.Compare(a.order(), b.order())
+	})
+}
+
+// unpackRecv unpacks a receiver type expression and returns its components: ptr indicates
+// whether rtyp is a pointer receiver, base is the receiver base type expression stripped
+// of its type parameters (if any), and tparams are its type parameter names, if any. The
+// type parameters are only unpacked if unpackParams is set. For instance, given the rtyp
+//
+//	*T[A, _]
+//
+// ptr is true, base is T, and tparams is [A, _] (assuming unpackParams is set).
+// Note that base may not be a *syntax.Name for erroneous programs.
+func (check *Checker) unpackRecv(rtyp syntax.Expr, unpackParams bool) (ptr bool, base syntax.Expr, tparams []*syntax.Name) {
+	// unpack receiver type
+	base = syntax.Unparen(rtyp)
+	if t, _ := base.(*syntax.Operation); t != nil && t.Op == syntax.Mul && t.Y == nil {
+		ptr = true
+		base = syntax.Unparen(t.X)
 	}
 
 	// unpack type parameters, if any
-	if ptyp, _ := rtyp.(*syntax.IndexExpr); ptyp != nil {
-		rtyp = ptyp.X
+	if ptyp, _ := base.(*syntax.IndexExpr); ptyp != nil {
+		base = ptyp.X
 		if unpackParams {
 			for _, arg := range syntax.UnpackListExpr(ptyp.Index) {
 				var par *syntax.Name
@@ -559,119 +563,84 @@ L: // unpack receiver type
 		}
 	}
 
-	// unpack receiver name
-	if name, _ := rtyp.(*syntax.Name); name != nil {
-		rname = name
-	}
-
 	return
 }
 
-// resolveBaseTypeName returns the non-alias base type name for typ, and whether
+// resolveBaseTypeName returns the non-alias base type name for the given name, and whether
 // there was a pointer indirection to get to it. The base type name must be declared
-// in package scope, and there can be at most one pointer indirection. If no such type
-// name exists, the returned base is nil.
-func (check *Checker) resolveBaseTypeName(seenPtr bool, typ syntax.Expr, fileScopes []*Scope) (ptr bool, base *TypeName) {
-	// Algorithm: Starting from a type expression, which may be a name,
-	// we follow that type through alias declarations until we reach a
-	// non-alias type name. If we encounter anything but pointer types or
-	// parentheses we're done. If we encounter more than one pointer type
-	// we're done.
-	ptr = seenPtr
+// in package scope, and there can be at most one pointer indirection. Traversals
+// through generic alias types are not permitted. If no such type name exists, the
+// returned base is nil.
+func (check *Checker) resolveBaseTypeName(ptr bool, name *syntax.Name) (ptr_ bool, base *TypeName) {
+	// Algorithm: Starting from name, which is expected to denote a type,
+	// we follow that type through non-generic alias declarations until
+	// we reach a non-alias type name.
 	var seen map[*TypeName]bool
-	for {
-		// check if we have a pointer type
-		// if pexpr, _ := typ.(*ast.StarExpr); pexpr != nil {
-		if pexpr, _ := typ.(*syntax.Operation); pexpr != nil && pexpr.Op == syntax.Mul && pexpr.Y == nil {
-			// if we've already seen a pointer, we're done
-			if ptr {
-				return false, nil
-			}
-			ptr = true
-			typ = syntax.Unparen(pexpr.X) // continue with pointer base type
-		}
-
-		// typ must be a name, or a C.name cgo selector.
-		var name string
-		switch typ := typ.(type) {
-		case *syntax.Name:
-			name = typ.Value
-		case *syntax.SelectorExpr:
-			// C.struct_foo is a valid type name for packages using cgo.
-			//
-			// Detect this case, and adjust name so that the correct TypeName is
-			// resolved below.
-			if ident, _ := typ.X.(*syntax.Name); ident != nil && ident.Value == "C" {
-				// Check whether "C" actually resolves to an import of "C", by looking
-				// in the appropriate file scope.
-				var obj Object
-				for _, scope := range fileScopes {
-					if scope.Contains(ident.Pos()) {
-						obj = scope.Lookup(ident.Value)
-					}
-				}
-				// If Config.go115UsesCgo is set, the typechecker will resolve Cgo
-				// selectors to their cgo name. We must do the same here.
-				if pname, _ := obj.(*PkgName); pname != nil {
-					if pname.imported.cgo { // only set if Config.go115UsesCgo is set
-						name = "_Ctype_" + typ.Sel.Value
-					}
-				}
-			}
-			if name == "" {
-				return false, nil
-			}
-		default:
-			return false, nil
-		}
-
+	for name != nil {
 		// name must denote an object found in the current package scope
 		// (note that dot-imported objects are not in the package scope!)
-		obj := check.pkg.scope.Lookup(name)
+		obj := check.pkg.scope.Lookup(name.Value)
 		if obj == nil {
-			return false, nil
+			break
 		}
 
 		// the object must be a type name...
 		tname, _ := obj.(*TypeName)
 		if tname == nil {
-			return false, nil
+			break
 		}
 
 		// ... which we have not seen before
 		if seen[tname] {
-			return false, nil
+			break
 		}
 
-		// we're done if tdecl defined tname as a new type
-		// (rather than an alias)
+		// we're done if tdecl describes a defined type (not an alias)
 		tdecl := check.objMap[tname].tdecl // must exist for objects in package scope
 		if !tdecl.Alias {
 			return ptr, tname
 		}
 
-		// otherwise, continue resolving
-		typ = tdecl.Type
+		// an alias must not be generic
+		// (importantly, we must not collect such methods - was https://go.dev/issue/70417)
+		if tdecl.TParamList != nil {
+			break
+		}
+
+		// otherwise, remember this type name and continue resolving
 		if seen == nil {
 			seen = make(map[*TypeName]bool)
 		}
 		seen[tname] = true
+
+		// The syntax parser strips unnecessary parentheses; call Unparen for consistency with go/types.
+		typ := syntax.Unparen(tdecl.Type)
+
+		// dereference a pointer type
+		if pexpr, _ := typ.(*syntax.Operation); pexpr != nil && pexpr.Op == syntax.Mul && pexpr.Y == nil {
+			// if we've already seen a pointer, we're done
+			if ptr {
+				break
+			}
+			ptr = true
+			typ = syntax.Unparen(pexpr.X) // continue with pointer base type
+		}
+
+		// After dereferencing, typ must be a locally defined type name.
+		// Referring to other packages (qualified identifiers) or going
+		// through instantiated types (index expressions) is not permitted,
+		// so we can ignore those.
+		name, _ = typ.(*syntax.Name)
 	}
+
+	// no base type found
+	return false, nil
 }
 
 // packageObjects typechecks all package objects, but not function bodies.
 func (check *Checker) packageObjects() {
-	// process package objects in source order for reproducible results
-	objList := make([]Object, len(check.objMap))
-	i := 0
-	for obj := range check.objMap {
-		objList[i] = obj
-		i++
-	}
-	sort.Sort(inSourceOrder(objList))
-
 	// add new methods to already type-checked types (from a prior Checker.Files call)
-	for _, obj := range objList {
+	for _, obj := range check.objList {
 		if obj, _ := obj.(*TypeName); obj != nil && obj.typ != nil {
 			check.collectMethods(obj)
 		}
@@ -694,8 +663,8 @@ func (check *Checker) packageObjects() {
 		// its Type is Invalid.
 		//
 		// Investigate and reenable this branch.
-		for _, obj := range objList {
-			check.objDecl(obj, nil)
+		for _, obj := range check.objList {
+			check.objDecl(obj)
 		}
 	} else {
 		// Without Alias nodes, we process non-alias type declarations first, followed by
@@ -706,12 +675,12 @@ func (check *Checker) packageObjects() {
 		var aliasList []*TypeName
 		var othersList []Object // everything that's not a type
 		// phase 1: non-alias type declarations
-		for _, obj := range objList {
+		for _, obj := range check.objList {
 			if tname, _ := obj.(*TypeName); tname != nil {
 				if check.objMap[tname].tdecl.Alias {
 					aliasList = append(aliasList, tname)
 				} else {
-					check.objDecl(obj, nil)
+					check.objDecl(obj)
 				}
 			} else {
 				othersList = append(othersList, obj)
@@ -719,11 +688,11 @@ func (check *Checker) packageObjects() {
 		}
 		// phase 2: alias type declarations
 		for _, obj := range aliasList {
-			check.objDecl(obj, nil)
+			check.objDecl(obj)
 		}
 		// phase 3: all other declarations
 		for _, obj := range othersList {
-			check.objDecl(obj, nil)
+			check.objDecl(obj)
 		}
 	}
 
@@ -734,13 +703,6 @@ func (check *Checker) packageObjects() {
 	check.methods = nil
 }
 
-// inSourceOrder implements the sort.Sort interface.
-type inSourceOrder []Object
-
-func (a inSourceOrder) Len() int           { return len(a) }
-func (a inSourceOrder) Less(i, j int) bool { return a[i].order() < a[j].order() }
-func (a inSourceOrder) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
-
 // unusedImports checks for unused imports.
 func (check *Checker) unusedImports() {
 	// If function bodies are not checked, packages' uses are likely missing - don't check.
@@ -753,7 +715,7 @@ func (check *Checker) unusedImports() {
 	// (initialization), use the blank identifier as explicit package name."
 
 	for _, obj := range check.imports {
-		if !obj.used && obj.name != "_" {
+		if obj.name != "_" && !check.usedPkgNames[obj] {
 			check.errorUnusedPkg(obj)
 		}
 	}
