@@ -9,7 +9,7 @@ package types2
 import (
 	"bytes"
 	"fmt"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"unicode/utf8"
@@ -308,7 +308,7 @@ func (w *typeWriter) typ(typ Type) {
 			w.error("unnamed type parameter")
 			break
 		}
-		if i := tparamIndex(w.tparams.list(), t); i >= 0 {
+		if i := slices.Index(w.tparams.list(), t); i >= 0 {
 			// The names of type parameters that are declared by the type being
 			// hashed are not part of the type identity. Replace them with a
 			// placeholder indicating their index.
@@ -338,6 +338,9 @@ func (w *typeWriter) typ(typ Type) {
 		if list := t.targs.list(); len(list) != 0 {
 			// instantiated type
 			w.typeList(list)
+		} else if w.ctxt == nil && t.TypeParams().Len() != 0 { // For type hashing, don't need to format the TypeParams
+			// parameterized type
+			w.tParamList(t.TypeParams().list())
 		}
 		if w.ctxt != nil {
 			// TODO(gri) do we need to print the alias type name, too?
@@ -379,7 +382,7 @@ func (w *typeWriter) typeSet(s *_TypeSet) {
 			newTypeHasher(&buf, w.ctxt).typ(term.typ)
 			termHashes = append(termHashes, buf.String())
 		}
-		sort.Strings(termHashes)
+		slices.Sort(termHashes)
 		if !first {
 			w.byte(';')
 		}
@@ -446,22 +449,25 @@ func (w *typeWriter) tuple(tup *Tuple, variadic bool) {
 			}
 			typ := v.typ
 			if variadic && i == len(tup.vars)-1 {
-				if s, ok := typ.(*Slice); ok {
+				if slice, ok := typ.(*Slice); ok {
 					w.string("...")
-					typ = s.elem
+					w.typ(slice.elem)
 				} else {
-					// special case:
-					// append(s, "foo"...) leads to signature func([]byte, string...)
-					if t, _ := under(typ).(*Basic); t == nil || t.kind != String {
-						w.error("expected string type")
-						continue
-					}
+					// append(slice, str...) entails various special
+					// cases, especially in conjunction with generics.
+					// str may be:
+					// - a string,
+					// - a TypeParam whose typeset includes string, or
+					// - a named []byte slice type B resulting from
+					//   a client instantiating append([]byte, T) at T=B.
+					// For such cases we use the irregular notation
+					// func([]byte, T...), with the dots after the type.
 					w.typ(typ)
 					w.string("...")
-					continue
 				}
+			} else {
+				w.typ(typ)
 			}
-			w.typ(typ)
 		}
 	}
 	w.byte(')')
