@@ -189,16 +189,16 @@ func findEnv(env []cfg.EnvVar, name string) string {
 }
 
 // ExtraEnvVars returns environment variables that should not leak into child processes.
-func ExtraEnvVars() []cfg.EnvVar {
+func ExtraEnvVars(loaderstate *modload.State) []cfg.EnvVar {
 	gomod := ""
-	modload.Init()
-	if modload.HasModRoot() {
-		gomod = modload.ModFilePath()
-	} else if modload.Enabled() {
+	modload.Init(loaderstate)
+	if loaderstate.HasModRoot() {
+		gomod = loaderstate.ModFilePath()
+	} else if loaderstate.Enabled() {
 		gomod = os.DevNull
 	}
-	modload.InitWorkfile()
-	gowork := modload.WorkFilePath()
+	loaderstate.InitWorkfile()
+	gowork := modload.WorkFilePath(loaderstate)
 	// As a special case, if a user set off explicitly, report that in GOWORK.
 	if cfg.Getenv("GOWORK") == "off" {
 		gowork = "off"
@@ -211,8 +211,8 @@ func ExtraEnvVars() []cfg.EnvVar {
 
 // ExtraEnvVarsCostly returns environment variables that should not leak into child processes
 // but are costly to evaluate.
-func ExtraEnvVarsCostly() []cfg.EnvVar {
-	b := work.NewBuilder("")
+func ExtraEnvVarsCostly(loaderstate *modload.State) []cfg.EnvVar {
+	b := work.NewBuilder("", loaderstate.VendorDirOrEmpty)
 	defer func() {
 		if err := b.Close(); err != nil {
 			base.Fatal(err)
@@ -272,6 +272,7 @@ func argKey(arg string) string {
 }
 
 func runEnv(ctx context.Context, cmd *base.Command, args []string) {
+	moduleLoaderState := modload.NewState()
 	if *envJson && *envU {
 		base.Fatalf("go: cannot use -json with -u")
 	}
@@ -306,7 +307,7 @@ func runEnv(ctx context.Context, cmd *base.Command, args []string) {
 	}
 
 	env := cfg.CmdEnv
-	env = append(env, ExtraEnvVars()...)
+	env = append(env, ExtraEnvVars(moduleLoaderState)...)
 
 	if err := fsys.Init(); err != nil {
 		base.Fatal(err)
@@ -336,8 +337,8 @@ func runEnv(ctx context.Context, cmd *base.Command, args []string) {
 		}
 	}
 	if needCostly {
-		work.BuildInit()
-		env = append(env, ExtraEnvVarsCostly()...)
+		work.BuildInit(moduleLoaderState)
+		env = append(env, ExtraEnvVarsCostly(moduleLoaderState)...)
 	}
 
 	if len(args) > 0 {
