@@ -384,6 +384,9 @@ func (r *codeRepo) convert(ctx context.Context, info *codehost.RevInfo, statVers
 			} else {
 				origin.TagPrefix = tags.Origin.TagPrefix
 				origin.TagSum = tags.Origin.TagSum
+				if tags.Origin.RepoSum != "" {
+					origin.RepoSum = tags.Origin.RepoSum
+				}
 			}
 		}
 	}()
@@ -1009,10 +1012,6 @@ func LegacyGoMod(modPath string) []byte {
 	return fmt.Appendf(nil, "module %s\n", modfile.AutoQuote(modPath))
 }
 
-func (r *codeRepo) modPrefix(rev string) string {
-	return r.modPath + "@" + rev
-}
-
 func (r *codeRepo) retractedVersions(ctx context.Context) (func(string) bool, error) {
 	vs, err := r.Versions(ctx, "")
 	if err != nil {
