@@ -5,11 +5,12 @@
 package ld
 
 import (
-	"cmd/internal/notsha256"
+	"cmd/internal/hash"
 	"cmd/internal/objabi"
 	"cmd/internal/sys"
 	"cmd/link/internal/loader"
 	"cmd/link/internal/sym"
+	"cmp"
 	"debug/elf"
 	"encoding/binary"
 	"encoding/hex"
@@ -18,7 +19,7 @@ import (
 	"os"
 	"path/filepath"
 	"runtime"
-	"sort"
+	"slices"
 	"strings"
 )
 
@@ -61,80 +62,44 @@ import (
  *
  */
 
-/*
- * ELF definitions that are independent of architecture or word size.
- */
+// ELF definitions that are independent of architecture or word size.
 
-/*
- * Note header.  The ".note" section contains an array of notes.  Each
- * begins with this header, aligned to a word boundary.  Immediately
- * following the note header is n_namesz bytes of name, padded to the
- * next word boundary.  Then comes n_descsz bytes of descriptor, again
- * padded to a word boundary.  The values of n_namesz and n_descsz do
- * not include the padding.
- */
-type elfNote struct {
-	nNamesz uint32
-	nDescsz uint32
-	nType   uint32
-}
+// We use the 64-bit data structures on both 32- and 64-bit machines
+// in order to write the code just once. The 64-bit data structure is
+// written in the 32-bit format on the 32-bit machines.
 
-/* For accessing the fields of r_info. */
-
-/* For constructing r_info from field values. */
-
-/*
- * Relocation types.
- */
-const (
-	ARM_MAGIC_TRAMP_NUMBER = 0x5c000003
-)
-
-/*
- * Symbol table entries.
- */
-
-/* For accessing the fields of st_info. */
-
-/* For constructing st_info from field values. */
-
-/* For accessing the fields of st_other. */
-
-/*
- * ELF header.
- */
+// ElfEhdr is the ELF file header.
 type ElfEhdr elf.Header64
 
-/*
- * Section header.
- */
+// ElfShdr is an ELF section table entry.
 type ElfShdr struct {
 	elf.Section64
+
+	// nameString is the section name as a string.
+	// This is not to be confused with Name,
+	// inherited from elf.Section64, which is an offset.
+	nameString string
+
+	// The section index, set by elfSortShdrs.
+	// Don't read this directly, use elfShdrShnum.
 	shnum elf.SectionIndex
+
+	// Because we don't compute the final section number
+	// until late in the link, when the link and info fields
+	// hold section indexes, we store pointers, and fetch
+	// the final section index when we write them out.
+	link *ElfShdr
+	info *ElfShdr
+
+	// We compute the section offsets of reloc sections
+	// after we create the ELF section header.
+	// This field lets us fetch the section offset and size.
+	relocSect *sym.Section
 }
 
-/*
- * Program header.
- */
+// ElfPhdr is the ELF program, or segment, header.
 type ElfPhdr elf.ProgHeader
 
-/* For accessing the fields of r_info. */
-
-/* For constructing r_info from field values. */
-
-/*
- * Symbol table entries.
- */
-
-/* For accessing the fields of st_info. */
-
-/* For constructing st_info from field values. */
-
-/* For accessing the fields of st_other. */
-
-/*
- * Go linker interface
- */
 const (
 	ELF64HDRSIZE  = 64
 	ELF64PHDRSIZE = 56
@@ -149,32 +114,13 @@ const (
 	ELF32RELSIZE  = 8
 )
 
-/*
- * The interface uses the 64-bit structures always,
- * to avoid code duplication.  The writers know how to
- * marshal a 32-bit representation from the 64-bit structure.
- */
+var elfstrdat []byte
 
-var elfstrdat, elfshstrdat []byte
-
-/*
- * Total amount of space to reserve at the start of the file
- * for Header, PHeaders, SHeaders, and interp.
- * May waste some.
- * On FreeBSD, cannot be larger than a page.
- */
-const (
-	ELFRESERVE = 4096
-)
-
-/*
- * We use the 64-bit data structures on both 32- and 64-bit machines
- * in order to write the code just once.  The 64-bit data structure is
- * written in the 32-bit format on the 32-bit machines.
- */
-const (
-	NSECT = 400
-)
+// ELFRESERVE is the total amount of space to reserve at the
+// start of the file for Header, PHeaders, SHeaders, and interp.
+// May waste some space.
+// On FreeBSD, cannot be larger than a page.
+const ELFRESERVE = 4096
 
 var (
 	Nelfsym = 1
@@ -184,9 +130,10 @@ var (
 	// target platform uses.
 	elfRelType string
 
-	ehdr ElfEhdr
-	phdr [NSECT]*ElfPhdr
-	shdr [NSECT]*ElfShdr
+	ehdr       ElfEhdr
+	phdr       = make([]*ElfPhdr, 0, 8)
+	shdr       = make([]*ElfShdr, 0, 64)
+	shdrSorted bool
 
 	interp string
 )
@@ -216,21 +163,10 @@ type ELFArch struct {
 	DynamicReadOnly bool
 }
 
-type Elfstring struct {
-	s   string
-	off int
-}
-
-var elfstr [100]Elfstring
-
-var nelfstr int
-
 var buildinfo []byte
 
-/*
-Initialize the global variable that describes the ELF header. It will be updated as
-we write section and prog headers.
-*/
+// Elfinit initializes the global ehdr variable that holds the ELF header.
+// It will be updated as write section and program headers.
 func Elfinit(ctxt *Link) {
 	ctxt.IsELF = true
 
@@ -244,28 +180,28 @@ func Elfinit(ctxt *Link) {
 	// 64-bit architectures
 	case sys.PPC64, sys.S390X:
 		if ctxt.Arch.ByteOrder == binary.BigEndian && ctxt.HeadType != objabi.Hopenbsd {
-			ehdr.Flags = 1 /* Version 1 ABI */
+			ehdr.Flags = 1 // Version 1 ABI
 		} else {
-			ehdr.Flags = 2 /* Version 2 ABI */
+			ehdr.Flags = 2 // Version 2 ABI
 		}
 		fallthrough
 	case sys.AMD64, sys.ARM64, sys.Loong64, sys.MIPS64, sys.RISCV64:
 		if ctxt.Arch.Family == sys.MIPS64 {
-			ehdr.Flags = 0x20000004 /* MIPS 3 CPIC */
+			ehdr.Flags = 0x20000004 // MIPS 3 CPIC
 		}
 		if ctxt.Arch.Family == sys.Loong64 {
-			ehdr.Flags = 0x43 /* DOUBLE_FLOAT, OBJABI_V1 */
+			ehdr.Flags = 0x43 // DOUBLE_FLOAT, OBJABI_V1
 		}
 		if ctxt.Arch.Family == sys.RISCV64 {
-			ehdr.Flags = 0x4 /* RISCV Float ABI Double */
+			ehdr.Flags = 0x4 // RISCV Float ABI Double
 		}
 		elf64 = true
 
-		ehdr.Phoff = ELF64HDRSIZE      /* Must be ELF64HDRSIZE: first PHdr must follow ELF header */
-		ehdr.Shoff = ELF64HDRSIZE      /* Will move as we add PHeaders */
-		ehdr.Ehsize = ELF64HDRSIZE     /* Must be ELF64HDRSIZE */
-		ehdr.Phentsize = ELF64PHDRSIZE /* Must be ELF64PHDRSIZE */
-		ehdr.Shentsize = ELF64SHDRSIZE /* Must be ELF64SHDRSIZE */
+		ehdr.Phoff = ELF64HDRSIZE      // Must be ELF64HDRSIZE: first PHdr must follow ELF header
+		ehdr.Shoff = ELF64HDRSIZE      // Will move as we add PHeaders
+		ehdr.Ehsize = ELF64HDRSIZE     // Must be ELF64HDRSIZE
+		ehdr.Phentsize = ELF64PHDRSIZE // Must be ELF64PHDRSIZE
+		ehdr.Shentsize = ELF64SHDRSIZE // Must be ELF64SHDRSIZE
 
 	// 32-bit architectures
 	case sys.ARM, sys.MIPS:
@@ -283,22 +219,21 @@ func Elfinit(ctxt *Link) {
 				ehdr.Flags = 0x5000002 // has entry point, Version5 EABI
 			}
 		} else if ctxt.Arch.Family == sys.MIPS {
-			ehdr.Flags = 0x50001004 /* MIPS 32 CPIC O32*/
+			ehdr.Flags = 0x50001004 // MIPS 32 CPIC O32
 		}
 		fallthrough
 	default:
 		ehdr.Phoff = ELF32HDRSIZE
-		/* Must be ELF32HDRSIZE: first PHdr must follow ELF header */
-		ehdr.Shoff = ELF32HDRSIZE      /* Will move as we add PHeaders */
-		ehdr.Ehsize = ELF32HDRSIZE     /* Must be ELF32HDRSIZE */
-		ehdr.Phentsize = ELF32PHDRSIZE /* Must be ELF32PHDRSIZE */
-		ehdr.Shentsize = ELF32SHDRSIZE /* Must be ELF32SHDRSIZE */
+		// Must be ELF32HDRSIZE: first PHdr must follow ELF header
+		ehdr.Shoff = ELF32HDRSIZE      // Will move as we add PHeaders
+		ehdr.Ehsize = ELF32HDRSIZE     // Must be ELF32HDRSIZE
+		ehdr.Phentsize = ELF32PHDRSIZE // Must be ELF32PHDRSIZE
+		ehdr.Shentsize = ELF32SHDRSIZE // Must be ELF32SHDRSIZE
 	}
 }
 
-// Make sure PT_LOAD is aligned properly and
-// that there is no gap,
-// correct ELF loaders will do this implicitly,
+// fixElfPhdr makes sure PT_LOAD is aligned properly and that there is no gap.
+// Correct ELF loaders will do this implicitly,
 // but buggy ELF loaders like the one in some
 // versions of QEMU and UPX won't.
 func fixElfPhdr(e *ElfPhdr) {
@@ -341,55 +276,181 @@ func elf32phdr(out *OutBuf, e *ElfPhdr) {
 	out.Write32(uint32(e.Align))
 }
 
+// elfShdrShnum returns the section index of an ElfShdr.
+func elfShdrShnum(e *ElfShdr) elf.SectionIndex {
+	if e.shnum == -1 {
+		Errorf("internal error: retrieved section index before it is set")
+		errorexit()
+	}
+	return e.shnum
+}
+
+// elfShdrOff returns the section offset for an ElfShdr.
+func elfShdrOff(e *ElfShdr) uint64 {
+	if e.relocSect != nil {
+		if e.Off != 0 {
+			Errorf("internal error: ElfShdr relocSect == %p Off == %d", e.relocSect, e.Off)
+			errorexit()
+		}
+		return e.relocSect.Reloff
+	}
+	return e.Off
+}
+
+// elfShdrSize returns the section size for an ElfShdr.
+func elfShdrSize(e *ElfShdr) uint64 {
+	if e.relocSect != nil {
+		if e.Size != 0 {
+			Errorf("internal error: ElfShdr relocSect == %p Size == %d", e.relocSect, e.Size)
+			errorexit()
+		}
+		return e.relocSect.Rellen
+	}
+	return e.Size
+}
+
+// elfShdrLink returns the link value for an ElfShdr.
+func elfShdrLink(e *ElfShdr) uint32 {
+	if e.link != nil {
+		if e.Link != 0 {
+			Errorf("internal error: ElfShdr link == %p Link == %d", e.link, e.Link)
+			errorexit()
+		}
+		return uint32(elfShdrShnum(e.link))
+	}
+	return e.Link
+}
+
+// elfShdrInfo returns the info value for an ElfShdr.
+func elfShdrInfo(e *ElfShdr) uint32 {
+	if e.info != nil {
+		if e.Info != 0 {
+			Errorf("internal error: ElfShdr info == %p Info == %d", e.info, e.Info)
+			errorexit()
+		}
+		return uint32(elfShdrShnum(e.info))
+	}
+	return e.Info
+}
+
 func elf64shdr(out *OutBuf, e *ElfShdr) {
 	out.Write32(e.Name)
-	out.Write32(uint32(e.Type))
-	out.Write64(uint64(e.Flags))
+	out.Write32(e.Type)
+	out.Write64(e.Flags)
 	out.Write64(e.Addr)
-	out.Write64(e.Off)
-	out.Write64(e.Size)
-	out.Write32(e.Link)
-	out.Write32(e.Info)
+	out.Write64(elfShdrOff(e))
+	out.Write64(elfShdrSize(e))
+	out.Write32(elfShdrLink(e))
+	out.Write32(elfShdrInfo(e))
 	out.Write64(e.Addralign)
 	out.Write64(e.Entsize)
 }
 
 func elf32shdr(out *OutBuf, e *ElfShdr) {
 	out.Write32(e.Name)
-	out.Write32(uint32(e.Type))
+	out.Write32(e.Type)
 	out.Write32(uint32(e.Flags))
 	out.Write32(uint32(e.Addr))
-	out.Write32(uint32(e.Off))
-	out.Write32(uint32(e.Size))
-	out.Write32(e.Link)
-	out.Write32(e.Info)
+	out.Write32(uint32(elfShdrOff(e)))
+	out.Write32(uint32(elfShdrSize(e)))
+	out.Write32(elfShdrLink(e))
+	out.Write32(elfShdrInfo(e))
 	out.Write32(uint32(e.Addralign))
 	out.Write32(uint32(e.Entsize))
 }
 
 func elfwriteshdrs(out *OutBuf) uint32 {
 	if elf64 {
-		for i := 0; i < int(ehdr.Shnum); i++ {
-			elf64shdr(out, shdr[i])
+		for _, sh := range shdr {
+			elf64shdr(out, sh)
 		}
-		return uint32(ehdr.Shnum) * ELF64SHDRSIZE
+		return uint32(len(shdr)) * ELF64SHDRSIZE
 	}
 
-	for i := 0; i < int(ehdr.Shnum); i++ {
-		elf32shdr(out, shdr[i])
+	for _, sh := range shdr {
+		elf32shdr(out, sh)
 	}
-	return uint32(ehdr.Shnum) * ELF32SHDRSIZE
+	return uint32(len(shdr)) * ELF32SHDRSIZE
 }
 
-func elfsetstring(ctxt *Link, s loader.Sym, str string, off int) {
-	if nelfstr >= len(elfstr) {
-		ctxt.Errorf(s, "too many elf strings")
-		errorexit()
+// elfSortShdrs sorts the section headers so that allocated sections
+// are first, in address order. This isn't required for correctness,
+// but it makes the ELF file easier for humans to read.
+// We only do this for an executable, not an object file.
+func elfSortShdrs(ctxt *Link) {
+	if ctxt.LinkMode != LinkExternal {
+		// Use [1:] to leave the empty section header zero in place.
+		slices.SortStableFunc(shdr[1:], func(a, b *ElfShdr) int {
+			isAllocated := func(h *ElfShdr) bool {
+				return elf.SectionFlag(h.Flags)&elf.SHF_ALLOC != 0
+			}
+			if isAllocated(a) {
+				if isAllocated(b) {
+					if r := cmp.Compare(a.Addr, b.Addr); r != 0 {
+						return r
+					}
+					// With same address, sort smallest
+					// section first.
+					return cmp.Compare(a.Size, b.Size)
+				}
+				// Allocated before unallocated.
+				return -1
+			}
+			if isAllocated(b) {
+				// Allocated before unallocated.
+				return 1
+			}
+			return 0
+		})
+	}
+	for i, h := range shdr {
+		h.shnum = elf.SectionIndex(i)
+	}
+	shdrSorted = true
+}
+
+// elfWriteShstrtab writes out the ELF section string table.
+// It also sets the Name field of the section headers.
+// It returns the length of the string table.
+func elfWriteShstrtab(ctxt *Link) uint32 {
+	// Map from section name to shstrtab offset.
+	m := make(map[string]uint32, len(shdr))
+
+	m[""] = 0
+	ctxt.Out.WriteByte(0)
+	off := uint32(1)
+
+	writeString := func(s string) {
+		m[s] = off
+		ctxt.Out.WriteString(s)
+		ctxt.Out.WriteByte(0)
+		off += uint32(len(s)) + 1
 	}
 
-	elfstr[nelfstr].s = str
-	elfstr[nelfstr].off = off
-	nelfstr++
+	// As a minor optimization, do the relocation sections first,
+	// as they may let us reuse the suffix.
+	// That is, the offset for ".text" can point into ".rel.text".
+	// We don't do a full suffix search as the relocation sections
+	// are likely to be the only match.
+	for _, sh := range shdr {
+		if suffix, ok := strings.CutPrefix(sh.nameString, elfRelType); ok {
+			if _, found := m[suffix]; !found {
+				m[suffix] = off + uint32(len(elfRelType))
+			}
+			writeString(sh.nameString)
+		}
+	}
+
+	for _, sh := range shdr {
+		if shOff, ok := m[sh.nameString]; ok {
+			sh.Name = shOff
+		} else {
+			sh.Name = off
+			writeString(sh.nameString)
+		}
+	}
+
+	return off
 }
 
 func elfwritephdrs(out *OutBuf) uint32 {
@@ -408,12 +469,8 @@ func elfwritephdrs(out *OutBuf) uint32 {
 
 func newElfPhdr() *ElfPhdr {
 	e := new(ElfPhdr)
-	if ehdr.Phnum >= NSECT {
-		Errorf(nil, "too many phdrs")
-	} else {
-		phdr[ehdr.Phnum] = e
-		ehdr.Phnum++
-	}
+	phdr = append(phdr, e)
+	ehdr.Phnum++
 	if elf64 {
 		ehdr.Shoff += ELF64PHDRSIZE
 	} else {
@@ -422,17 +479,17 @@ func newElfPhdr() *ElfPhdr {
 	return e
 }
 
-func newElfShdr(name int64) *ElfShdr {
-	e := new(ElfShdr)
-	e.Name = uint32(name)
-	e.shnum = elf.SectionIndex(ehdr.Shnum)
-	if ehdr.Shnum >= NSECT {
-		Errorf(nil, "too many shdrs")
-	} else {
-		shdr[ehdr.Shnum] = e
-		ehdr.Shnum++
+func newElfShdr(name string) *ElfShdr {
+	if shdrSorted {
+		Errorf("internal error: creating a section header after they were sorted")
+		errorexit()
 	}
 
+	e := &ElfShdr{
+		nameString: name,
+		shnum:      -1, // make invalid for now, set by elfSortShdrs
+	}
+	shdr = append(shdr, e)
 	return e
 }
 
@@ -442,9 +499,9 @@ func getElfEhdr() *ElfEhdr {
 
 func elf64writehdr(out *OutBuf) uint32 {
 	out.Write(ehdr.Ident[:])
-	out.Write16(uint16(ehdr.Type))
-	out.Write16(uint16(ehdr.Machine))
-	out.Write32(uint32(ehdr.Version))
+	out.Write16(ehdr.Type)
+	out.Write16(ehdr.Machine)
+	out.Write32(ehdr.Version)
 	out.Write64(ehdr.Entry)
 	out.Write64(ehdr.Phoff)
 	out.Write64(ehdr.Shoff)
@@ -460,9 +517,9 @@ func elf64writehdr(out *OutBuf) uint32 {
 
 func elf32writehdr(out *OutBuf) uint32 {
 	out.Write(ehdr.Ident[:])
-	out.Write16(uint16(ehdr.Type))
-	out.Write16(uint16(ehdr.Machine))
-	out.Write32(uint32(ehdr.Version))
+	out.Write16(ehdr.Type)
+	out.Write16(ehdr.Machine)
+	out.Write32(ehdr.Version)
 	out.Write32(uint32(ehdr.Entry))
 	out.Write32(uint32(ehdr.Phoff))
 	out.Write32(uint32(ehdr.Shoff))
@@ -483,7 +540,10 @@ func elfwritehdr(out *OutBuf) uint32 {
 	return elf32writehdr(out)
 }
 
-/* Taken directly from the definition document for ELF64. */
+// elfhash is the dynamic symbol hash function .
+// This is taken directly from the definition document for ELF,
+// except that instead of finishing with "h &^= g" we just write
+// "h &= 0xfffffff", which results in the same value.
 func elfhash(name string) uint32 {
 	var h uint32
 	for i := 0; i < len(name); i++ {
@@ -578,6 +638,8 @@ func elfMipsAbiFlags(sh *ElfShdr, startva uint64, resoff uint64) int {
 	return n
 }
 
+// elfWriteMipsAbiFlags writes the .MIPS.abiflags section.
+//
 // Layout is given by this C definition:
 //
 //	typedef struct
@@ -668,7 +730,7 @@ const (
 	ELF_NOTE_NETBSD_NAMESZ  = 7
 	ELF_NOTE_NETBSD_DESCSZ  = 4
 	ELF_NOTE_NETBSD_TAG     = 1
-	ELF_NOTE_NETBSD_VERSION = 700000000 /* NetBSD 7.0 */
+	ELF_NOTE_NETBSD_VERSION = 700000000 // NetBSD 7.0
 )
 
 var ELF_NOTE_NETBSD_NAME = []byte("NetBSD\x00")
@@ -805,14 +867,23 @@ func elfwritefreebsdsig(out *OutBuf) int {
 	return int(sh.Size)
 }
 
-func addbuildinfo(val string) {
+func addbuildinfo(ctxt *Link) {
+	val := *flagHostBuildid
+	if val == "" || val == "none" {
+		return
+	}
 	if val == "gobuildid" {
 		buildID := *flagBuildid
 		if buildID == "" {
 			Exitf("-B gobuildid requires a Go build ID supplied via -buildid")
 		}
 
-		hashedBuildID := notsha256.Sum256([]byte(buildID))
+		if ctxt.IsDarwin() {
+			buildinfo = uuidFromGoBuildId(buildID)
+			return
+		}
+
+		hashedBuildID := hash.Sum32([]byte(buildID))
 		buildinfo = hashedBuildID[:20]
 
 		return
@@ -821,11 +892,13 @@ func addbuildinfo(val string) {
 	if !strings.HasPrefix(val, "0x") {
 		Exitf("-B argument must start with 0x: %s", val)
 	}
-
 	ov := val
 	val = val[2:]
 
-	const maxLen = 32
+	maxLen := 32
+	if ctxt.IsDarwin() {
+		maxLen = 16
+	}
 	if hex.DecodedLen(len(val)) > maxLen {
 		Exitf("-B option too long (max %d digits): %s", maxLen, ov)
 	}
@@ -1056,11 +1129,17 @@ func elfdynhash(ctxt *Link) {
 	}
 
 	s = ldr.CreateSymForUpdate(".dynamic", 0)
-	if ctxt.BuildMode == BuildModePIE {
-		// https://github.com/bminor/glibc/blob/895ef79e04a953cac1493863bcae29ad85657ee1/elf/elf.h#L986
-		const DTFLAGS_1_PIE = 0x08000000
-		Elfwritedynent(ctxt.Arch, s, elf.DT_FLAGS_1, uint64(DTFLAGS_1_PIE))
+
+	var dtFlags1 elf.DynFlag1
+	if *flagBindNow {
+		dtFlags1 |= elf.DF_1_NOW
+		Elfwritedynent(ctxt.Arch, s, elf.DT_FLAGS, uint64(elf.DF_BIND_NOW))
 	}
+	if ctxt.BuildMode == BuildModePIE {
+		dtFlags1 |= elf.DF_1_PIE
+	}
+	Elfwritedynent(ctxt.Arch, s, elf.DT_FLAGS_1, uint64(dtFlags1))
+
 	elfverneed = nfile
 	if elfverneed != 0 {
 		elfWriteDynEntSym(ctxt, s, elf.DT_VERNEED, gnuVersionR.Sym())
@@ -1107,6 +1186,7 @@ func elfphload(seg *sym.Segment) *ElfPhdr {
 func elfphrelro(seg *sym.Segment) {
 	ph := newElfPhdr()
 	ph.Type = elf.PT_GNU_RELRO
+	ph.Flags = elf.PF_R
 	ph.Vaddr = seg.Vaddr
 	ph.Paddr = seg.Vaddr
 	ph.Memsz = seg.Length
@@ -1115,37 +1195,20 @@ func elfphrelro(seg *sym.Segment) {
 	ph.Align = uint64(*FlagRound)
 }
 
+// elfshname finds or creates a section given its name.
 func elfshname(name string) *ElfShdr {
-	for i := 0; i < nelfstr; i++ {
-		if name != elfstr[i].s {
-			continue
+	for _, sh := range shdr {
+		if sh.nameString == name {
+			return sh
 		}
-		off := elfstr[i].off
-		for i = 0; i < int(ehdr.Shnum); i++ {
-			sh := shdr[i]
-			if sh.Name == uint32(off) {
-				return sh
-			}
-		}
-		return newElfShdr(int64(off))
 	}
-	Exitf("cannot find elf name %s", name)
-	return nil
+	return newElfShdr(name)
 }
 
-// Create an ElfShdr for the section with name.
-// Create a duplicate if one already exists with that name.
+// elfshnamedup creates a new section with a given name.
+// If there is an existing section with this name, it creates a duplicate.
 func elfshnamedup(name string) *ElfShdr {
-	for i := 0; i < nelfstr; i++ {
-		if name == elfstr[i].s {
-			off := elfstr[i].off
-			return newElfShdr(int64(off))
-		}
-	}
-
-	Errorf(nil, "cannot find elf name %s", name)
-	errorexit()
-	return nil
+	return newElfShdr(name)
 }
 
 func elfshalloc(sect *sym.Section) *ElfShdr {
@@ -1177,7 +1240,7 @@ func elfshbits(linkmode LinkMode, sect *sym.Section) *ElfShdr {
 			// list note). The real fix is probably to define new values
 			// for Symbol.Type corresponding to mapped and unmapped notes
 			// and handle them in dodata().
-			Errorf(nil, "sh.Type == SHT_NOTE in elfshbits when linking internally")
+			Errorf("sh.Type == SHT_NOTE in elfshbits when linking internally")
 		}
 		sh.Addralign = uint64(sect.Align)
 		sh.Size = sect.Length
@@ -1232,11 +1295,12 @@ func elfshbits(linkmode LinkMode, sect *sym.Section) *ElfShdr {
 
 func elfshreloc(arch *sys.Arch, sect *sym.Section) *ElfShdr {
 	// If main section is SHT_NOBITS, nothing to relocate.
-	// Also nothing to relocate in .shstrtab or notes.
+	// Also nothing to relocate in .shstrtab or notes or .gopclntab.
 	if sect.Vaddr >= sect.Seg.Vaddr+sect.Seg.Filelen {
 		return nil
 	}
-	if sect.Name == ".shstrtab" || sect.Name == ".tbss" {
+	switch sect.Name {
+	case ".shstrtab", ".tbss", ".gopclntab":
 		return nil
 	}
 	if sect.Elfsect.(*ElfShdr).Type == uint32(elf.SHT_NOTE) {
@@ -1253,7 +1317,7 @@ func elfshreloc(arch *sys.Arch, sect *sym.Section) *ElfShdr {
 	// its own .rela.text.
 
 	if sect.Name == ".text" {
-		if sh.Info != 0 && sh.Info != uint32(sect.Elfsect.(*ElfShdr).shnum) {
+		if sh.info != nil && sh.info != sect.Elfsect.(*ElfShdr) {
 			sh = elfshnamedup(elfRelType + sect.Name)
 		}
 	}
@@ -1263,10 +1327,9 @@ func elfshreloc(arch *sys.Arch, sect *sym.Section) *ElfShdr {
 	if typ == elf.SHT_RELA {
 		sh.Entsize += uint64(arch.RegSize)
 	}
-	sh.Link = uint32(elfshname(".symtab").shnum)
-	sh.Info = uint32(sect.Elfsect.(*ElfShdr).shnum)
-	sh.Off = sect.Reloff
-	sh.Size = sect.Rellen
+	sh.link = elfshname(".symtab")
+	sh.info = sect.Elfsect.(*ElfShdr)
+	sh.relocSect = sect
 	sh.Addralign = uint64(arch.RegSize)
 	return sh
 }
@@ -1361,7 +1424,7 @@ func elfEmitReloc(ctxt *Link) {
 	for i := 0; i < len(Segdwarf.Sections); i++ {
 		sect := Segdwarf.Sections[i]
 		si := dwarfp[i]
-		if si.secSym() != loader.Sym(sect.Sym) ||
+		if si.secSym() != sect.Sym ||
 			ctxt.loader.SymSect(si.secSym()) != sect {
 			panic("inconsistency between dwarfp and Segdwarf")
 		}
@@ -1397,142 +1460,12 @@ func addgonote(ctxt *Link, sectionName string, tag uint32, desc []byte) {
 func (ctxt *Link) doelf() {
 	ldr := ctxt.loader
 
-	/* predefine strings we need for section headers */
-
-	addshstr := func(s string) int {
-		off := len(elfshstrdat)
-		elfshstrdat = append(elfshstrdat, s...)
-		elfshstrdat = append(elfshstrdat, 0)
-		return off
-	}
-
-	shstrtabAddstring := func(s string) {
-		off := addshstr(s)
-		elfsetstring(ctxt, 0, s, int(off))
-	}
-
-	shstrtabAddstring("")
-	shstrtabAddstring(".text")
-	shstrtabAddstring(".noptrdata")
-	shstrtabAddstring(".data")
-	shstrtabAddstring(".bss")
-	shstrtabAddstring(".noptrbss")
-	shstrtabAddstring(".go.fuzzcntrs")
-	shstrtabAddstring(".go.buildinfo")
-	if ctxt.IsMIPS() {
-		shstrtabAddstring(".MIPS.abiflags")
-		shstrtabAddstring(".gnu.attributes")
-	}
-
-	// generate .tbss section for dynamic internal linker or external
-	// linking, so that various binutils could correctly calculate
-	// PT_TLS size. See https://golang.org/issue/5200.
-	if !*FlagD || ctxt.IsExternal() {
-		shstrtabAddstring(".tbss")
-	}
-	if ctxt.IsNetbsd() {
-		shstrtabAddstring(".note.netbsd.ident")
-		if *flagRace {
-			shstrtabAddstring(".note.netbsd.pax")
-		}
-	}
-	if ctxt.IsOpenbsd() {
-		shstrtabAddstring(".note.openbsd.ident")
-	}
-	if ctxt.IsFreebsd() {
-		shstrtabAddstring(".note.tag")
-	}
-	if len(buildinfo) > 0 {
-		shstrtabAddstring(".note.gnu.build-id")
-	}
-	if *flagBuildid != "" {
-		shstrtabAddstring(".note.go.buildid")
-	}
-	shstrtabAddstring(".elfdata")
-	shstrtabAddstring(".rodata")
-	// See the comment about data.rel.ro.FOO section names in data.go.
-	relro_prefix := ""
-	if ctxt.UseRelro() {
-		shstrtabAddstring(".data.rel.ro")
-		relro_prefix = ".data.rel.ro"
-	}
-	shstrtabAddstring(relro_prefix + ".typelink")
-	shstrtabAddstring(relro_prefix + ".itablink")
-	shstrtabAddstring(relro_prefix + ".gosymtab")
-	shstrtabAddstring(relro_prefix + ".gopclntab")
-
 	if ctxt.IsExternal() {
 		*FlagD = true
-
-		shstrtabAddstring(elfRelType + ".text")
-		shstrtabAddstring(elfRelType + ".rodata")
-		shstrtabAddstring(elfRelType + relro_prefix + ".typelink")
-		shstrtabAddstring(elfRelType + relro_prefix + ".itablink")
-		shstrtabAddstring(elfRelType + relro_prefix + ".gosymtab")
-		shstrtabAddstring(elfRelType + relro_prefix + ".gopclntab")
-		shstrtabAddstring(elfRelType + ".noptrdata")
-		shstrtabAddstring(elfRelType + ".data")
-		if ctxt.UseRelro() {
-			shstrtabAddstring(elfRelType + ".data.rel.ro")
-		}
-		shstrtabAddstring(elfRelType + ".go.buildinfo")
-		if ctxt.IsMIPS() {
-			shstrtabAddstring(elfRelType + ".MIPS.abiflags")
-			shstrtabAddstring(elfRelType + ".gnu.attributes")
-		}
-
-		// add a .note.GNU-stack section to mark the stack as non-executable
-		shstrtabAddstring(".note.GNU-stack")
-
-		if ctxt.IsShared() {
-			shstrtabAddstring(".note.go.abihash")
-			shstrtabAddstring(".note.go.pkg-list")
-			shstrtabAddstring(".note.go.deps")
-		}
 	}
 
-	hasinitarr := ctxt.linkShared
-
-	/* shared library initializer */
-	switch ctxt.BuildMode {
-	case BuildModeCArchive, BuildModeCShared, BuildModeShared, BuildModePlugin:
-		hasinitarr = true
-	}
-
-	if hasinitarr {
-		shstrtabAddstring(".init_array")
-		shstrtabAddstring(elfRelType + ".init_array")
-	}
-
-	if !*FlagS {
-		shstrtabAddstring(".symtab")
-		shstrtabAddstring(".strtab")
-	}
-	if !*FlagW {
-		dwarfaddshstrings(ctxt, shstrtabAddstring)
-	}
-
-	shstrtabAddstring(".shstrtab")
-
-	if !*FlagD { /* -d suppresses dynamic loader format */
-		shstrtabAddstring(".interp")
-		shstrtabAddstring(".hash")
-		shstrtabAddstring(".got")
-		if ctxt.IsPPC64() {
-			shstrtabAddstring(".glink")
-		}
-		shstrtabAddstring(".got.plt")
-		shstrtabAddstring(".dynamic")
-		shstrtabAddstring(".dynsym")
-		shstrtabAddstring(".dynstr")
-		shstrtabAddstring(elfRelType)
-		shstrtabAddstring(elfRelType + ".plt")
-
-		shstrtabAddstring(".plt")
-		shstrtabAddstring(".gnu.version")
-		shstrtabAddstring(".gnu.version_r")
-
-		/* dynamic symbol table - first entry all zeros */
+	if !*FlagD { // -d suppresses dynamic loader format
+		// dynamic symbol table - first entry all zeros
 		dynsym := ldr.CreateSymForUpdate(".dynsym", 0)
 
 		dynsym.SetType(sym.SELFROSECT)
@@ -1542,7 +1475,7 @@ func (ctxt *Link) doelf() {
 			dynsym.SetSize(dynsym.Size() + ELF32SYMSIZE)
 		}
 
-		/* dynamic string table */
+		// dynamic string table
 		dynstr := ldr.CreateSymForUpdate(".dynstr", 0)
 
 		dynstr.SetType(sym.SELFROSECT)
@@ -1550,26 +1483,34 @@ func (ctxt *Link) doelf() {
 			dynstr.Addstring("")
 		}
 
-		/* relocation table */
+		// relocation table
 		s := ldr.CreateSymForUpdate(elfRelType, 0)
 		s.SetType(sym.SELFROSECT)
 
-		/* global offset table */
+		// global offset table
 		got := ldr.CreateSymForUpdate(".got", 0)
-		got.SetType(sym.SELFGOT) // writable
+		if ctxt.UseRelro() {
+			got.SetType(sym.SELFRELROSECT)
+		} else {
+			got.SetType(sym.SELFGOT) // writable
+		}
 
-		/* ppc64 glink resolver */
+		// ppc64 glink resolver
 		if ctxt.IsPPC64() {
 			s := ldr.CreateSymForUpdate(".glink", 0)
 			s.SetType(sym.SELFRXSECT)
 		}
 
-		/* hash */
+		// hash
 		hash := ldr.CreateSymForUpdate(".hash", 0)
 		hash.SetType(sym.SELFROSECT)
 
 		gotplt := ldr.CreateSymForUpdate(".got.plt", 0)
-		gotplt.SetType(sym.SELFSECT) // writable
+		if ctxt.UseRelro() && *flagBindNow {
+			gotplt.SetType(sym.SELFRELROSECT)
+		} else {
+			gotplt.SetType(sym.SELFSECT) // writable
+		}
 
 		plt := ldr.CreateSymForUpdate(".plt", 0)
 		if ctxt.IsPPC64() {
@@ -1589,11 +1530,14 @@ func (ctxt *Link) doelf() {
 		s = ldr.CreateSymForUpdate(".gnu.version_r", 0)
 		s.SetType(sym.SELFROSECT)
 
-		/* define dynamic elf table */
+		// define dynamic elf table
 		dynamic := ldr.CreateSymForUpdate(".dynamic", 0)
-		if thearch.ELF.DynamicReadOnly {
+		switch {
+		case thearch.ELF.DynamicReadOnly:
 			dynamic.SetType(sym.SELFROSECT)
-		} else {
+		case ctxt.UseRelro():
+			dynamic.SetType(sym.SELFRELROSECT)
+		default:
 			dynamic.SetType(sym.SELFSECT)
 		}
 
@@ -1603,9 +1547,7 @@ func (ctxt *Link) doelf() {
 		}
 		thearch.ELF.SetupPLT(ctxt, ctxt.loader, plt, gotplt, dynamic.Sym())
 
-		/*
-		 * .dynamic table
-		 */
+		// .dynamic table
 		elfWriteDynEntSym(ctxt, dynamic, elf.DT_HASH, hash.Sym())
 
 		elfWriteDynEntSym(ctxt, dynamic, elf.DT_SYMTAB, dynsym.Sym())
@@ -1659,10 +1601,11 @@ func (ctxt *Link) doelf() {
 		sb.SetType(sym.SRODATA)
 		ldr.SetAttrSpecial(s, true)
 		sb.SetReachable(true)
-		sb.SetSize(notsha256.Size)
-
-		sort.Sort(byPkg(ctxt.Library))
-		h := notsha256.New()
+		sb.SetSize(hash.Size32)
+		slices.SortFunc(ctxt.Library, func(a, b *sym.Library) int {
+			return strings.Compare(a.Pkg, b.Pkg)
+		})
+		h := hash.New32()
 		for _, l := range ctxt.Library {
 			h.Write(l.Fingerprint[:])
 		}
@@ -1709,7 +1652,7 @@ func (ctxt *Link) doelf() {
 	}
 }
 
-// Do not write DT_NULL.  elfdynhash will finish it.
+// shsym fills in fields of sh where s contains the contents of the section.
 func shsym(sh *ElfShdr, ldr *loader.Loader, s loader.Sym) {
 	if s == 0 {
 		panic("bad symbol in shsym2")
@@ -1732,7 +1675,7 @@ func phsh(ph *ElfPhdr, sh *ElfShdr) {
 }
 
 func Asmbelfsetup() {
-	/* This null SHdr must appear before all others */
+	// This null SHdr must appear before all others.
 	elfshname("")
 
 	for _, sect := range Segtext.Sections {
@@ -1764,19 +1707,6 @@ func asmbElf(ctxt *Link) {
 	var symo int64
 	symo = int64(Segdwarf.Fileoff + Segdwarf.Filelen)
 	symo = Rnd(symo, int64(ctxt.Arch.PtrSize))
-	ctxt.Out.SeekSet(symo)
-	if *FlagS {
-		ctxt.Out.Write(elfshstrdat)
-	} else {
-		ctxt.Out.SeekSet(symo)
-		asmElfSym(ctxt)
-		ctxt.Out.Write(elfstrdat)
-		ctxt.Out.Write(elfshstrdat)
-		if ctxt.IsExternal() {
-			elfEmitReloc(ctxt)
-		}
-	}
-	ctxt.Out.SeekSet(0)
 
 	ldr := ctxt.loader
 	eh := getElfEhdr()
@@ -1840,7 +1770,7 @@ func asmbElf(ctxt *Link) {
 		phsh(getpnote(), sh)
 	}
 	if ctxt.LinkMode == LinkExternal {
-		/* skip program headers */
+		// skip program headers
 		eh.Phoff = 0
 
 		eh.Phentsize = 0
@@ -1864,7 +1794,7 @@ func asmbElf(ctxt *Link) {
 		goto elfobj
 	}
 
-	/* program header info */
+	// program header info
 	pph = newElfPhdr()
 
 	pph.Type = elf.PT_PHDR
@@ -1874,10 +1804,8 @@ func asmbElf(ctxt *Link) {
 	pph.Paddr = uint64(*FlagTextAddr) - uint64(HEADR) + pph.Off
 	pph.Align = uint64(*FlagRound)
 
-	/*
-	 * PHDR must be in a loaded segment. Adjust the text
-	 * segment boundaries downwards to include it.
-	 */
+	// PHDR must be in a loaded segment. Adjust the text
+	// segment boundaries downwards to include it.
 	{
 		o := int64(Segtext.Vaddr - pph.Vaddr)
 		Segtext.Vaddr -= uint64(o)
@@ -1887,8 +1815,8 @@ func asmbElf(ctxt *Link) {
 		Segtext.Filelen += uint64(o)
 	}
 
-	if !*FlagD { /* -d suppresses dynamic loader format */
-		/* interpreter */
+	if !*FlagD { // -d suppresses dynamic loader format
+		// interpreter
 		sh := elfshname(".interp")
 
 		sh.Type = uint32(elf.SHT_PROGBITS)
@@ -1992,7 +1920,7 @@ func asmbElf(ctxt *Link) {
 	}
 	elfphload(&Segdata)
 
-	/* Dynamic linking sections */
+	// Dynamic linking sections
 	if !*FlagD {
 		sh := elfshname(".dynsym")
 		sh.Type = uint32(elf.SHT_DYNSYM)
@@ -2003,9 +1931,9 @@ func asmbElf(ctxt *Link) {
 			sh.Entsize = ELF32SYMSIZE
 		}
 		sh.Addralign = uint64(ctxt.Arch.RegSize)
-		sh.Link = uint32(elfshname(".dynstr").shnum)
+		sh.link = elfshname(".dynstr")
 
-		// sh.info is the index of first non-local symbol (number of local symbols)
+		// sh.Info is the index of first non-local symbol (number of local symbols)
 		s := ldr.Lookup(".dynsym", 0)
 		i := uint32(0)
 		for sub := s; sub != 0; sub = ldr.SubSym(sub) {
@@ -2028,7 +1956,7 @@ func asmbElf(ctxt *Link) {
 			sh.Type = uint32(elf.SHT_GNU_VERSYM)
 			sh.Flags = uint64(elf.SHF_ALLOC)
 			sh.Addralign = 2
-			sh.Link = uint32(elfshname(".dynsym").shnum)
+			sh.link = elfshname(".dynsym")
 			sh.Entsize = 2
 			shsym(sh, ldr, ldr.Lookup(".gnu.version", 0))
 
@@ -2037,7 +1965,7 @@ func asmbElf(ctxt *Link) {
 			sh.Flags = uint64(elf.SHF_ALLOC)
 			sh.Addralign = uint64(ctxt.Arch.RegSize)
 			sh.Info = uint32(elfverneed)
-			sh.Link = uint32(elfshname(".dynstr").shnum)
+			sh.link = elfshname(".dynstr")
 			shsym(sh, ldr, ldr.Lookup(".gnu.version_r", 0))
 		}
 
@@ -2047,8 +1975,8 @@ func asmbElf(ctxt *Link) {
 			sh.Flags = uint64(elf.SHF_ALLOC)
 			sh.Entsize = ELF64RELASIZE
 			sh.Addralign = uint64(ctxt.Arch.RegSize)
-			sh.Link = uint32(elfshname(".dynsym").shnum)
-			sh.Info = uint32(elfshname(".plt").shnum)
+			sh.link = elfshname(".dynsym")
+			sh.info = elfshname(".plt")
 			shsym(sh, ldr, ldr.Lookup(".rela.plt", 0))
 
 			sh = elfshname(".rela")
@@ -2056,7 +1984,7 @@ func asmbElf(ctxt *Link) {
 			sh.Flags = uint64(elf.SHF_ALLOC)
 			sh.Entsize = ELF64RELASIZE
 			sh.Addralign = 8
-			sh.Link = uint32(elfshname(".dynsym").shnum)
+			sh.link = elfshname(".dynsym")
 			shsym(sh, ldr, ldr.Lookup(".rela", 0))
 		} else {
 			sh := elfshname(".rel.plt")
@@ -2064,7 +1992,7 @@ func asmbElf(ctxt *Link) {
 			sh.Flags = uint64(elf.SHF_ALLOC)
 			sh.Entsize = ELF32RELSIZE
 			sh.Addralign = 4
-			sh.Link = uint32(elfshname(".dynsym").shnum)
+			sh.link = elfshname(".dynsym")
 			shsym(sh, ldr, ldr.Lookup(".rel.plt", 0))
 
 			sh = elfshname(".rel")
@@ -2072,7 +2000,7 @@ func asmbElf(ctxt *Link) {
 			sh.Flags = uint64(elf.SHF_ALLOC)
 			sh.Entsize = ELF32RELSIZE
 			sh.Addralign = 4
-			sh.Link = uint32(elfshname(".dynsym").shnum)
+			sh.link = elfshname(".dynsym")
 			shsym(sh, ldr, ldr.Lookup(".rel", 0))
 		}
 
@@ -2127,26 +2055,24 @@ func asmbElf(ctxt *Link) {
 		sh.Flags = uint64(elf.SHF_ALLOC)
 		sh.Entsize = 4
 		sh.Addralign = uint64(ctxt.Arch.RegSize)
-		sh.Link = uint32(elfshname(".dynsym").shnum)
+		sh.link = elfshname(".dynsym")
 		shsym(sh, ldr, ldr.Lookup(".hash", 0))
 
-		/* sh and elf.PT_DYNAMIC for .dynamic section */
+		// sh and elf.PT_DYNAMIC for .dynamic section
 		sh = elfshname(".dynamic")
 
 		sh.Type = uint32(elf.SHT_DYNAMIC)
 		sh.Flags = uint64(elf.SHF_ALLOC + elf.SHF_WRITE)
 		sh.Entsize = 2 * uint64(ctxt.Arch.RegSize)
 		sh.Addralign = uint64(ctxt.Arch.RegSize)
-		sh.Link = uint32(elfshname(".dynstr").shnum)
+		sh.link = elfshname(".dynstr")
 		shsym(sh, ldr, ldr.Lookup(".dynamic", 0))
 		ph := newElfPhdr()
 		ph.Type = elf.PT_DYNAMIC
 		ph.Flags = elf.PF_R + elf.PF_W
 		phsh(ph, sh)
 
-		/*
-		 * Thread-local storage segment (really just size).
-		 */
+		// Thread-local storage segment (really just size).
 		tlssize := uint64(0)
 		for _, sect := range Segdata.Sections {
 			if sect.Name == ".tbss" {
@@ -2167,6 +2093,10 @@ func asmbElf(ctxt *Link) {
 		ph.Type = elf.PT_GNU_STACK
 		ph.Flags = elf.PF_W + elf.PF_R
 		ph.Align = uint64(ctxt.Arch.RegSize)
+	} else if ctxt.HeadType == objabi.Hopenbsd {
+		ph := newElfPhdr()
+		ph.Type = elf.PT_OPENBSD_NOBTCFI
+		ph.Flags = elf.PF_X
 	} else if ctxt.HeadType == objabi.Hsolaris {
 		ph := newElfPhdr()
 		ph.Type = elf.PT_SUNWSTACK
@@ -2174,11 +2104,8 @@ func asmbElf(ctxt *Link) {
 	}
 
 elfobj:
-	sh := elfshname(".shstrtab")
-	eh.Shstrndx = uint16(sh.shnum)
-
 	if ctxt.IsMIPS() {
-		sh = elfshname(".MIPS.abiflags")
+		sh := elfshname(".MIPS.abiflags")
 		sh.Type = uint32(elf.SHT_MIPS_ABIFLAGS)
 		sh.Flags = uint64(elf.SHF_ALLOC)
 		sh.Addralign = 8
@@ -2244,6 +2171,25 @@ elfobj:
 		sh.Flags = 0
 	}
 
+	elfSortShdrs(ctxt)
+
+	sh := elfshname(".shstrtab")
+	eh.Shstrndx = uint16(elfShdrShnum(sh))
+
+	var shstrtabLen uint32
+	ctxt.Out.SeekSet(symo)
+	if *FlagS {
+		shstrtabLen = elfWriteShstrtab(ctxt)
+	} else {
+		asmElfSym(ctxt)
+		ctxt.Out.Write(elfstrdat)
+		shstrtabLen = elfWriteShstrtab(ctxt)
+		if ctxt.IsExternal() {
+			elfEmitReloc(ctxt)
+		}
+	}
+	ctxt.Out.SeekSet(0)
+
 	var shstroff uint64
 	if !*FlagS {
 		sh := elfshname(".symtab")
@@ -2252,7 +2198,7 @@ elfobj:
 		sh.Size = uint64(symSize)
 		sh.Addralign = uint64(ctxt.Arch.RegSize)
 		sh.Entsize = 8 + 2*uint64(ctxt.Arch.RegSize)
-		sh.Link = uint32(elfshname(".strtab").shnum)
+		sh.link = elfshname(".strtab")
 		sh.Info = uint32(elfglobalsymndx)
 
 		sh = elfshname(".strtab")
@@ -2268,10 +2214,10 @@ elfobj:
 	sh = elfshname(".shstrtab")
 	sh.Type = uint32(elf.SHT_STRTAB)
 	sh.Off = shstroff
-	sh.Size = uint64(len(elfshstrdat))
+	sh.Size = uint64(shstrtabLen)
 	sh.Addralign = 1
 
-	/* Main header */
+	// Main header
 	copy(eh.Ident[:], elf.ELFMAG)
 
 	var osabi elf.OSABI
@@ -2318,6 +2264,11 @@ elfobj:
 		pph.Memsz = pph.Filesz
 	}
 
+	if len(shdr) >= 0xffff {
+		Errorf("too many ELF sections")
+	}
+	eh.Shnum = uint16(len(shdr))
+
 	ctxt.Out.SeekSet(0)
 	a := int64(0)
 	a += int64(elfwritehdr(ctxt.Out))
@@ -2352,13 +2303,13 @@ elfobj:
 	}
 
 	if a > elfreserve {
-		Errorf(nil, "ELFRESERVE too small: %d > %d with %d text sections", a, elfreserve, numtext)
+		Errorf("ELFRESERVE too small: %d > %d with %d text sections", a, elfreserve, numtext)
 	}
 
 	// Verify the amount of space allocated for the elf header is sufficient.  The file offsets are
 	// already computed in layout, so we could spill into another section.
 	if a > int64(HEADR) {
-		Errorf(nil, "HEADR too small: %d > %d with %d text sections", a, HEADR, numtext)
+		Errorf("HEADR too small: %d > %d with %d text sections", a, HEADR, numtext)
 	}
 }
 
@@ -2376,35 +2327,34 @@ func elfadddynsym(ldr *loader.Loader, target *Target, syms *ArchSyms, s loader.S
 	d.AddUint32(target.Arch, uint32(dstru.Addstring(name)))
 
 	if elf64 {
-
-		/* type */
+		// type
 		var t uint8
 
-		if cgoexp && st == sym.STEXT {
+		if cgoexp && st.IsText() {
 			t = elf.ST_INFO(elf.STB_GLOBAL, elf.STT_FUNC)
 		} else {
 			t = elf.ST_INFO(elf.STB_GLOBAL, elf.STT_OBJECT)
 		}
 		d.AddUint8(t)
 
-		/* reserved */
+		// reserved
 		d.AddUint8(0)
 
-		/* section where symbol is defined */
+		// section where symbol is defined
 		if st == sym.SDYNIMPORT {
 			d.AddUint16(target.Arch, uint16(elf.SHN_UNDEF))
 		} else {
 			d.AddUint16(target.Arch, 1)
 		}
 
-		/* value */
+		// value
 		if st == sym.SDYNIMPORT {
 			d.AddUint64(target.Arch, 0)
 		} else {
 			d.AddAddrPlus(target.Arch, s, 0)
 		}
 
-		/* size of object */
+		// size of object
 		d.AddUint64(target.Arch, uint64(len(ldr.Data(s))))
 
 		dil := ldr.SymDynimplib(s)
@@ -2416,23 +2366,23 @@ func elfadddynsym(ldr *loader.Loader, target *Target, syms *ArchSyms, s loader.S
 		}
 	} else {
 
-		/* value */
+		// value
 		if st == sym.SDYNIMPORT {
 			d.AddUint32(target.Arch, 0)
 		} else {
 			d.AddAddrPlus(target.Arch, s, 0)
 		}
 
-		/* size of object */
+		// size of object
 		d.AddUint32(target.Arch, uint32(len(ldr.Data(s))))
 
-		/* type */
+		// type
 		var t uint8
 
 		// TODO(mwhudson): presumably the behavior should actually be the same on both arm and 386.
-		if target.Arch.Family == sys.I386 && cgoexp && st == sym.STEXT {
+		if target.Arch.Family == sys.I386 && cgoexp && st.IsText() {
 			t = elf.ST_INFO(elf.STB_GLOBAL, elf.STT_FUNC)
-		} else if target.Arch.Family == sys.ARM && cgoeDynamic && st == sym.STEXT {
+		} else if target.Arch.Family == sys.ARM && cgoeDynamic && st.IsText() {
 			t = elf.ST_INFO(elf.STB_GLOBAL, elf.STT_FUNC)
 		} else {
 			t = elf.ST_INFO(elf.STB_GLOBAL, elf.STT_OBJECT)
@@ -2440,7 +2390,7 @@ func elfadddynsym(ldr *loader.Loader, target *Target, syms *ArchSyms, s loader.S
 		d.AddUint8(t)
 		d.AddUint8(0)
 
-		/* shndx */
+		// shndx
 		if st == sym.SDYNIMPORT {
 			d.AddUint16(target.Arch, uint16(elf.SHN_UNDEF))
 		} else {
