@@ -94,14 +94,8 @@ func TestNoDuplicateNeededEntries(t *testing.T) {
 	t.Parallel()
 
 	dir := t.TempDir()
-
-	wd, err := os.Getwd()
-	if err != nil {
-		t.Fatalf("Failed to get working directory: %v", err)
-	}
-
 	path := filepath.Join(dir, "x")
-	argv := []string{"build", "-o", path, filepath.Join(wd, "testdata", "issue39256")}
+	argv := []string{"build", "-o", path, "./testdata/issue39256"}
 	out, err := testenv.Command(t, testenv.GoToolPath(t), argv...).CombinedOutput()
 	if err != nil {
 		t.Fatalf("Build failure: %s\n%s\n", err, string(out))
@@ -284,7 +278,8 @@ func TestElfBindNow(t *testing.T) {
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
 			if test.mustInternalLink {
-				testenv.MustInternalLink(t, test.mustHaveCGO)
+				// N.B. none of the tests pass -asan/-msan/-asan.
+				testenv.MustInternalLink(t, testenv.SpecialBuildTypes{Cgo: test.mustHaveCGO})
 			}
 			if test.mustHaveCGO {
 				testenv.MustHaveCGO(t)
@@ -413,7 +408,7 @@ func TestElfBindNow(t *testing.T) {
 }
 
 // This program is intended to be just big/complicated enough that
-// we wind up with decent-sized .data.rel.ro.{typelink,itablink,gopclntab}
+// we wind up with decent-sized .data.rel.ro.{typelink,itablink}
 // sections.
 const ifacecallsProg = `
 package main
@@ -595,7 +590,7 @@ func TestRelroSectionOverlapIssue67261(t *testing.T) {
 		} else {
 			// Non-empty output indicates failure, as mentioned above.
 			if len(string(sout)) != 0 {
-				t.Errorf("unexpected outut from %s:\n%s\n", sprog, string(sout))
+				t.Errorf("unexpected output from %s:\n%s\n", sprog, string(sout))
 			}
 		}
 		rcmd := testenv.Command(t, filepath.Join(dir, targ))
