@@ -242,7 +242,7 @@ func parseArmAttributes(e binary.ByteOrder, data []byte) (found bool, ehdrFlags
 // object, and the returned ehdrFlags contains what this Load function computes.
 // TODO: find a better place for this logic.
 func Load(l *loader.Loader, arch *sys.Arch, localSymVersion int, f *bio.Reader, pkg string, length int64, pn string, initEhdrFlags uint32) (textp []loader.Sym, ehdrFlags uint32, err error) {
-	errorf := func(str string, args ...interface{}) ([]loader.Sym, uint32, error) {
+	errorf := func(str string, args ...any) ([]loader.Sym, uint32, error) {
 		return nil, 0, fmt.Errorf("loadelf: %s: %v", pn, fmt.Sprintf(str, args...))
 	}
 
@@ -602,6 +602,19 @@ func Load(l *loader.Loader, arch *sys.Arch, localSymVersion int, f *bio.Reader,
 					// See https://sourceware.org/bugzilla/show_bug.cgi?id=21809
 					continue
 				}
+
+				if arch.Family == sys.RISCV64 &&
+					(strings.HasPrefix(elfsym.name, "$d") || strings.HasPrefix(elfsym.name, "$x")) {
+					// Ignore RISC-V mapping symbols, which
+					// are similar to ARM64's case.
+					// See issue 73591.
+					continue
+				}
+
+				if arch.Family == sys.Loong64 && (strings.HasPrefix(elfsym.name, ".L") || elfsym.name == "L0\001") {
+					// Symbols generated by the relax feature of gcc and binutils on loong64.
+					continue
+				}
 			}
 
 			if strings.HasPrefix(elfsym.name, ".Linfo_string") {
@@ -609,7 +622,7 @@ func Load(l *loader.Loader, arch *sys.Arch, localSymVersion int, f *bio.Reader,
 				continue
 			}
 
-			if strings.HasPrefix(elfsym.name, ".LASF") || strings.HasPrefix(elfsym.name, ".LLRL") || strings.HasPrefix(elfsym.name, ".LLST") {
+			if strings.HasPrefix(elfsym.name, ".LASF") || strings.HasPrefix(elfsym.name, ".LLRL") || strings.HasPrefix(elfsym.name, ".LLST") || strings.HasPrefix(elfsym.name, ".LVUS") {
 				// gcc on s390x and riscv64 does this.
 				continue
 			}
@@ -682,6 +695,12 @@ func Load(l *loader.Loader, arch *sys.Arch, localSymVersion int, f *bio.Reader,
 			l.SetAttrOnList(s, true)
 			textp = append(textp, s)
 			for ss := l.SubSym(s); ss != 0; ss = l.SubSym(ss) {
+				if arch.Family == sys.Loong64 && (strings.HasPrefix(l.SymName(ss), ".L") || l.SymName(ss) == "L0\001") {
+					// Symbols generated by the relax feature of gcc and binutils on loong64.
+					// We ignore them here because there are too many symbols of this type,
+					// resulting in insufficient space in findfunctable.
+					continue
+				}
 				if l.AttrOnList(ss) {
 					return errorf("symbol %s listed multiple times",
 						l.SymName(ss))
@@ -766,12 +785,12 @@ func Load(l *loader.Loader, arch *sys.Arch, localSymVersion int, f *bio.Reader,
 				rSym = 0
 			} else {
 				var elfsym ElfSym
-				if err := readelfsym(l, arch, elfobj, int(symIdx), &elfsym, 0, 0); err != nil {
+				if err := readelfsym(l, arch, elfobj, symIdx, &elfsym, 0, 0); err != nil {
 					return errorf("malformed elf file: %v", err)
 				}
 				elfsym.sym = symbols[symIdx]
 				if elfsym.sym == 0 {
-					return errorf("malformed elf file: %s#%d: reloc of invalid sym #%d %s shndx=%d type=%d", l.SymName(sect.sym), j, int(symIdx), elfsym.name, elfsym.shndx, elfsym.type_)
+					return errorf("malformed elf file: %s#%d: reloc of invalid sym #%d %s shndx=%d type=%d", l.SymName(sect.sym), j, symIdx, elfsym.name, elfsym.shndx, elfsym.type_)
 				}
 
 				rSym = elfsym.sym
@@ -835,7 +854,7 @@ func elfmap(elfobj *ElfObj, sect *ElfSect) (err error) {
 	}
 
 	elfobj.f.MustSeek(int64(uint64(elfobj.base)+sect.off), 0)
-	sect.base, sect.readOnlyMem, err = elfobj.f.Slice(uint64(sect.size))
+	sect.base, sect.readOnlyMem, err = elfobj.f.Slice(sect.size)
 	if err != nil {
 		return fmt.Errorf("short read: %v", err)
 	}
@@ -1018,7 +1037,14 @@ func relSize(arch *sys.Arch, pn string, elftype uint32) (uint8, uint8, error) {
 		MIPS64 | uint32(elf.R_MIPS_PC32)<<16:
 		return 4, 4, nil
 
+	// These are informational annotations to assist linker optimizations.
+	case LOONG64 | uint32(elf.R_LARCH_ALIGN)<<16,
+		LOONG64 | uint32(elf.R_LARCH_RELAX)<<16:
+		return 0, 0, nil
+
 	case LOONG64 | uint32(elf.R_LARCH_ADD8)<<16,
+		LOONG64 | uint32(elf.R_LARCH_ADD6)<<16,
+		LOONG64 | uint32(elf.R_LARCH_SUB6)<<16,
 		LOONG64 | uint32(elf.R_LARCH_SUB8)<<16:
 		return 1, 1, nil
 
@@ -1032,14 +1058,22 @@ func relSize(arch *sys.Arch, pn string, elftype uint32) (uint8, uint8, error) {
 		LOONG64 | uint32(elf.R_LARCH_ADD32)<<16,
 		LOONG64 | uint32(elf.R_LARCH_SUB24)<<16,
 		LOONG64 | uint32(elf.R_LARCH_SUB32)<<16,
+		LOONG64 | uint32(elf.R_LARCH_B16)<<16,
+		LOONG64 | uint32(elf.R_LARCH_B21)<<16,
 		LOONG64 | uint32(elf.R_LARCH_B26)<<16,
-		LOONG64 | uint32(elf.R_LARCH_32_PCREL)<<16:
+		LOONG64 | uint32(elf.R_LARCH_PCALA_HI20)<<16,
+		LOONG64 | uint32(elf.R_LARCH_PCALA_LO12)<<16,
+		LOONG64 | uint32(elf.R_LARCH_GOT_PC_HI20)<<16,
+		LOONG64 | uint32(elf.R_LARCH_GOT_PC_LO12)<<16,
+		LOONG64 | uint32(elf.R_LARCH_32_PCREL)<<16,
+		LOONG64 | uint32(elf.R_LARCH_PCREL20_S2)<<16:
 		return 4, 4, nil
 
 	case LOONG64 | uint32(elf.R_LARCH_64)<<16,
 		LOONG64 | uint32(elf.R_LARCH_ADD64)<<16,
 		LOONG64 | uint32(elf.R_LARCH_SUB64)<<16,
-		LOONG64 | uint32(elf.R_LARCH_64_PCREL)<<16:
+		LOONG64 | uint32(elf.R_LARCH_64_PCREL)<<16,
+		LOONG64 | uint32(elf.R_LARCH_CALL36)<<16:
 		return 8, 8, nil
 
 	case S390X | uint32(elf.R_390_8)<<16:
@@ -1146,6 +1180,7 @@ func relSize(arch *sys.Arch, pn string, elftype uint32) (uint8, uint8, error) {
 		RISCV64 | uint32(elf.R_RISCV_SET32)<<16,
 		RISCV64 | uint32(elf.R_RISCV_SUB32)<<16,
 		RISCV64 | uint32(elf.R_RISCV_32_PCREL)<<16,
+		RISCV64 | uint32(elf.R_RISCV_JAL)<<16,
 		RISCV64 | uint32(elf.R_RISCV_RELAX)<<16:
 		return 4, 4, nil
 
