@@ -694,7 +694,8 @@ func testInvalidPrivateKeys(t *testing.T, curve elliptic.Curve) {
 			t.Errorf("ParseRawPrivateKey accepted short key")
 		}
 
-		b = append(b, make([]byte, (curve.Params().BitSize+7)/8)...)
+		b = make([]byte, (curve.Params().BitSize+7)/8)
+		b = append(b, []byte{1, 2, 3}...)
 		if _, err := ParseRawPrivateKey(curve, b); err == nil {
 			t.Errorf("ParseRawPrivateKey accepted long key")
 		}
