@@ -11,7 +11,6 @@ import (
 	"go/ast"
 	"go/token"
 	"internal/lazyregexp"
-	"path"
 	"slices"
 	"strconv"
 	"strings"
@@ -75,6 +74,9 @@ func Examples(testFiles ...*ast.File) []*Example {
 			if params := f.Type.Params; len(params.List) != 0 {
 				continue // function has params; not a valid example
 			}
+			if results := f.Type.Results; results != nil && len(results.List) != 0 {
+				continue // function has results; not a valid example
+			}
 			if f.Body == nil { // ast.File.Body nil dereference (see issue 28044)
 				continue
 			}
@@ -221,7 +223,7 @@ func playExample(file *ast.File, f *ast.FuncDecl) *ast.File {
 			// because the package syscall/js is not available in the playground.
 			return nil
 		}
-		n := path.Base(p)
+		n := assumedPackageName(p)
 		if s.Name != nil {
 			n = s.Name.Name
 			switch n {
@@ -238,7 +240,7 @@ func playExample(file *ast.File, f *ast.FuncDecl) *ast.File {
 			spec := *s
 			path := *s.Path
 			spec.Path = &path
-			spec.Path.ValuePos = groupStart(&spec)
+			updateBasicLitPos(spec.Path, groupStart(&spec))
 			namedImports = append(namedImports, &spec)
 			delete(unresolved, n)
 		}
@@ -720,3 +722,14 @@ func isExampleSuffix(s string) bool {
 	r, size := utf8.DecodeRuneInString(s)
 	return size > 0 && unicode.IsLower(r)
 }
+
+// updateBasicLitPos updates lit.Pos,
+// ensuring that lit.End is displaced by the same amount.
+// (See https://go.dev/issue/76395.)
+func updateBasicLitPos(lit *ast.BasicLit, pos token.Pos) {
+	len := lit.End() - lit.Pos()
+	lit.ValuePos = pos
+	if lit.ValueEnd.IsValid() {
+		lit.ValueEnd = pos + len
+	}
+}
