@@ -18,7 +18,11 @@ func (T) M() {}
 var _ = rand.Int
 var _ = crand.Reader
 
-type G[T any] struct{ x T }
+type G[T any] struct{ X T }
 
-func (g G[T]) M1() {}
+func (g G[T]) M1()  {}
 func (g *G[T]) M2() {}
+
+type I interface {
+	F()
+}
