@@ -137,7 +137,7 @@ func (check *Checker) structType(styp *Struct, e *ast.StructType) {
 			// Because we have a name, typ must be of the form T or *T, where T is the name
 			// of a (named or alias) type, and t (= deref(typ)) must be the type of T.
 			// We must delay this check to the end because we don't want to instantiate
-			// (via under(t)) a possibly incomplete type.
+			// (via t.Underlying()) a possibly incomplete type.
 
 			// for use in the closure below
 			embeddedTyp := typ
@@ -145,7 +145,7 @@ func (check *Checker) structType(styp *Struct, e *ast.StructType) {
 
 			check.later(func() {
 				t, isPtr := deref(embeddedTyp)
-				switch u := under(t).(type) {
+				switch u := t.Underlying().(type) {
 				case *Basic:
 					if !isValid(t) {
 						// error was reported before
