@@ -6,6 +6,8 @@
 
 package codegen
 
+import "math"
+
 // This file contains codegen tests related to arithmetic
 // simplifications and optimizations on float types.
 // For codegen tests on integer types, see arithmetic.go.
@@ -15,38 +17,42 @@ package codegen
 // --------------------- //
 
 func Mul2(f float64) float64 {
-	// 386/sse2:"ADDSD",-"MULSD"
-	// amd64:"ADDSD",-"MULSD"
-	// arm/7:"ADDD",-"MULD"
-	// arm64:"FADDD",-"FMULD"
-	// ppc64x:"FADD",-"FMUL"
-	// riscv64:"FADDD",-"FMULD"
+	// 386/sse2:"ADDSD" -"MULSD"
+	// amd64:"ADDSD" -"MULSD"
+	// arm/7:"ADDD" -"MULD"
+	// arm64:"FADDD" -"FMULD"
+	// loong64:"ADDD" -"MULD"
+	// ppc64x:"FADD" -"FMUL"
+	// riscv64:"FADDD" -"FMULD"
 	return f * 2.0
 }
 
 func DivPow2(f1, f2, f3 float64) (float64, float64, float64) {
-	// 386/sse2:"MULSD",-"DIVSD"
-	// amd64:"MULSD",-"DIVSD"
-	// arm/7:"MULD",-"DIVD"
-	// arm64:"FMULD",-"FDIVD"
-	// ppc64x:"FMUL",-"FDIV"
-	// riscv64:"FMULD",-"FDIVD"
+	// 386/sse2:"MULSD" -"DIVSD"
+	// amd64:"MULSD" -"DIVSD"
+	// arm/7:"MULD" -"DIVD"
+	// arm64:"FMULD" -"FDIVD"
+	// loong64:"MULD" -"DIVD"
+	// ppc64x:"FMUL" -"FDIV"
+	// riscv64:"FMULD" -"FDIVD"
 	x := f1 / 16.0
 
-	// 386/sse2:"MULSD",-"DIVSD"
-	// amd64:"MULSD",-"DIVSD"
-	// arm/7:"MULD",-"DIVD"
-	// arm64:"FMULD",-"FDIVD"
-	// ppc64x:"FMUL",-"FDIVD"
-	// riscv64:"FMULD",-"FDIVD"
+	// 386/sse2:"MULSD" -"DIVSD"
+	// amd64:"MULSD" -"DIVSD"
+	// arm/7:"MULD" -"DIVD"
+	// arm64:"FMULD" -"FDIVD"
+	// loong64:"MULD" -"DIVD"
+	// ppc64x:"FMUL" -"FDIVD"
+	// riscv64:"FMULD" -"FDIVD"
 	y := f2 / 0.125
 
-	// 386/sse2:"ADDSD",-"DIVSD",-"MULSD"
-	// amd64:"ADDSD",-"DIVSD",-"MULSD"
-	// arm/7:"ADDD",-"MULD",-"DIVD"
-	// arm64:"FADDD",-"FMULD",-"FDIVD"
-	// ppc64x:"FADD",-"FMUL",-"FDIV"
-	// riscv64:"FADDD",-"FMULD",-"FDIVD"
+	// 386/sse2:"ADDSD" -"DIVSD" -"MULSD"
+	// amd64:"ADDSD" -"DIVSD" -"MULSD"
+	// arm/7:"ADDD" -"MULD" -"DIVD"
+	// arm64:"FADDD" -"FMULD" -"FDIVD"
+	// loong64:"ADDD" -"MULD" -"DIVD"
+	// ppc64x:"FADD" -"FMUL" -"FDIV"
+	// riscv64:"FADDD" -"FMULD" -"FDIVD"
 	z := f3 / 0.5
 
 	return x, y, z
@@ -69,67 +75,67 @@ func indexStore(b0 []float64, b1 float64, idx int) {
 // ----------- //
 
 func FusedAdd32(x, y, z float32) float32 {
-	// s390x:"FMADDS\t"
-	// ppc64x:"FMADDS\t"
+	// s390x:"FMADDS "
+	// ppc64x:"FMADDS "
 	// arm64:"FMADDS"
-	// loong64:"FMADDF\t"
-	// riscv64:"FMADDS\t"
-	// amd64/v3:"VFMADD231SS\t"
+	// loong64:"FMADDF "
+	// riscv64:"FMADDS "
+	// amd64/v3:"VFMADD231SS "
 	return x*y + z
 }
 
 func FusedSub32_a(x, y, z float32) float32 {
-	// s390x:"FMSUBS\t"
-	// ppc64x:"FMSUBS\t"
-	// riscv64:"FMSUBS\t"
-	// loong64:"FMSUBF\t"
+	// s390x:"FMSUBS "
+	// ppc64x:"FMSUBS "
+	// riscv64:"FMSUBS "
+	// loong64:"FMSUBF "
 	return x*y - z
 }
 
 func FusedSub32_b(x, y, z float32) float32 {
 	// arm64:"FMSUBS"
-	// loong64:"FNMSUBF\t"
-	// riscv64:"FNMSUBS\t"
+	// loong64:"FNMSUBF "
+	// riscv64:"FNMSUBS "
 	return z - x*y
 }
 
 func FusedAdd64(x, y, z float64) float64 {
-	// s390x:"FMADD\t"
-	// ppc64x:"FMADD\t"
+	// s390x:"FMADD "
+	// ppc64x:"FMADD "
 	// arm64:"FMADDD"
-	// loong64:"FMADDD\t"
-	// riscv64:"FMADDD\t"
-	// amd64/v3:"VFMADD231SD\t"
+	// loong64:"FMADDD "
+	// riscv64:"FMADDD "
+	// amd64/v3:"VFMADD231SD "
 	return x*y + z
 }
 
 func FusedSub64_a(x, y, z float64) float64 {
-	// s390x:"FMSUB\t"
-	// ppc64x:"FMSUB\t"
-	// riscv64:"FMSUBD\t"
-	// loong64:"FMSUBD\t"
+	// s390x:"FMSUB "
+	// ppc64x:"FMSUB "
+	// riscv64:"FMSUBD "
+	// loong64:"FMSUBD "
 	return x*y - z
 }
 
 func FusedSub64_b(x, y, z float64) float64 {
 	// arm64:"FMSUBD"
-	// loong64:"FNMSUBD\t"
-	// riscv64:"FNMSUBD\t"
+	// loong64:"FNMSUBD "
+	// riscv64:"FNMSUBD "
 	return z - x*y
 }
 
 func Cmp(f float64) bool {
-	// arm64:"FCMPD","(BGT|BLE|BMI|BPL)",-"CSET\tGT",-"CBZ"
+	// arm64:"FCMPD" "(BGT|BLE|BMI|BPL)" -"CSET GT" -"CBZ"
 	return f > 4 || f < -4
 }
 
 func CmpZero64(f float64) bool {
-	// s390x:"LTDBR",-"FCMPU"
+	// s390x:"LTDBR" -"FCMPU"
 	return f <= 0
 }
 
 func CmpZero32(f float32) bool {
-	// s390x:"LTEBR",-"CEBR"
+	// s390x:"LTEBR" -"CEBR"
 	return f <= 0
 }
 
@@ -172,6 +178,7 @@ func Float64Min(a, b float64) float64 {
 	// riscv64:"FMIN"
 	// ppc64/power9:"XSMINJDP"
 	// ppc64/power10:"XSMINJDP"
+	// s390x: "WFMINDB"
 	return min(a, b)
 }
 
@@ -182,6 +189,7 @@ func Float64Max(a, b float64) float64 {
 	// riscv64:"FMAX"
 	// ppc64/power9:"XSMAXJDP"
 	// ppc64/power10:"XSMAXJDP"
+	// s390x: "WFMAXDB"
 	return max(a, b)
 }
 
@@ -192,6 +200,7 @@ func Float32Min(a, b float32) float32 {
 	// riscv64:"FMINS"
 	// ppc64/power9:"XSMINJDP"
 	// ppc64/power10:"XSMINJDP"
+	// s390x: "WFMINSB"
 	return min(a, b)
 }
 
@@ -202,6 +211,7 @@ func Float32Max(a, b float32) float32 {
 	// riscv64:"FMAXS"
 	// ppc64/power9:"XSMAXJDP"
 	// ppc64/power10:"XSMAXJDP"
+	// s390x: "WFMAXSB"
 	return max(a, b)
 }
 
@@ -209,22 +219,44 @@ func Float32Max(a, b float32) float32 {
 //  Constant Optimizations  //
 // ------------------------ //
 
+func Float32ConstantZero() float32 {
+	// arm64:"FMOVS ZR,"
+	return 0.0
+}
+
+func Float32ConstantChipFloat() float32 {
+	// arm64:"FMOVS [$]\\(2\\.25\\),"
+	return 2.25
+}
+
 func Float32Constant() float32 {
-	// ppc64x/power8:"FMOVS\t[$]f32\\.42440000\\(SB\\)"
-	// ppc64x/power9:"FMOVS\t[$]f32\\.42440000\\(SB\\)"
-	// ppc64x/power10:"XXSPLTIDP\t[$]1111752704,"
+	// arm64:"FMOVS [$]f32\\.42440000\\(SB\\)"
+	// ppc64x/power8:"FMOVS [$]f32\\.42440000\\(SB\\)"
+	// ppc64x/power9:"FMOVS [$]f32\\.42440000\\(SB\\)"
+	// ppc64x/power10:"XXSPLTIDP [$]1111752704,"
 	return 49.0
 }
 
+func Float64ConstantZero() float64 {
+	// arm64:"FMOVD ZR,"
+	return 0.0
+}
+
+func Float64ConstantChipFloat() float64 {
+	// arm64:"FMOVD [$]\\(2\\.25\\),"
+	return 2.25
+}
+
 func Float64Constant() float64 {
-	// ppc64x/power8:"FMOVD\t[$]f64\\.4048800000000000\\(SB\\)"
-	// ppc64x/power9:"FMOVD\t[$]f64\\.4048800000000000\\(SB\\)"
-	// ppc64x/power10:"XXSPLTIDP\t[$]1111752704,"
+	// arm64:"FMOVD [$]f64\\.4048800000000000\\(SB\\)"
+	// ppc64x/power8:"FMOVD [$]f64\\.4048800000000000\\(SB\\)"
+	// ppc64x/power9:"FMOVD [$]f64\\.4048800000000000\\(SB\\)"
+	// ppc64x/power10:"XXSPLTIDP [$]1111752704,"
 	return 49.0
 }
 
 func Float32DenormalConstant() float32 {
-	// ppc64x:"FMOVS\t[$]f32\\.00400000\\(SB\\)"
+	// ppc64x:"FMOVS [$]f32\\.00400000\\(SB\\)"
 	return 0x1p-127
 }
 
@@ -232,15 +264,52 @@ func Float32DenormalConstant() float32 {
 // denormal float32 value. On ppc64x, denormal values cannot
 // be used with XXSPLTIDP.
 func Float64DenormalFloat32Constant() float64 {
-	// ppc64x:"FMOVD\t[$]f64\\.3800000000000000\\(SB\\)"
+	// ppc64x:"FMOVD [$]f64\\.3800000000000000\\(SB\\)"
 	return 0x1p-127
 }
 
-func Float64ConstantStore(p *float64) {
-	// amd64: "MOVQ\t[$]4617801906721357038"
-	*p = 5.432
-}
 func Float32ConstantStore(p *float32) {
-	// amd64: "MOVL\t[$]1085133554"
+	// amd64:"MOVL [$]1085133554"
+	// riscv64: "MOVF [$]f32.40add2f2"
 	*p = 5.432
 }
+
+func Float64ConstantStore(p *float64) {
+	// amd64: "MOVQ [$]4617801906721357038"
+	// riscv64: "MOVD [$]f64.4015ba5e353f7cee"
+	*p = 5.432
+}
+
+// ------------------------ //
+//  Subnormal tests         //
+// ------------------------ //
+
+func isSubnormal(x float64) bool {
+	// riscv64:"FCLASSD" -"FABSD"
+	return math.Abs(x) < 2.2250738585072014e-308
+}
+
+func isNormal(x float64) bool {
+	// riscv64:"FCLASSD" -"FABSD"
+	return math.Abs(x) >= 0x1p-1022
+}
+
+func isPosSubnormal(x float64) bool {
+	// riscv64:"FCLASSD"
+	return x > 0 && x < 2.2250738585072014e-308
+}
+
+func isNegSubnormal(x float64) bool {
+	// riscv64:"FCLASSD"
+	return x < 0 && x > -0x1p-1022
+}
+
+func isPosNormal(x float64) bool {
+	// riscv64:"FCLASSD"
+	return x >= 2.2250738585072014e-308
+}
+
+func isNegNormal(x float64) bool {
+	// riscv64:"FCLASSD"
+	return x <= -2.2250738585072014e-308
+}
