@@ -9,7 +9,7 @@ import "math"
 func f0a(a []int) int {
 	x := 0
 	for i := range a { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		x += a[i] // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += a[i] // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -17,7 +17,7 @@ func f0a(a []int) int {
 func f0b(a []int) int {
 	x := 0
 	for i := range a { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		b := a[i:] // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		b := a[i:] // ERROR "Proved IsSliceInBounds$"
 		x += b[0]
 	}
 	return x
@@ -26,8 +26,8 @@ func f0b(a []int) int {
 func f0c(a []int) int {
 	x := 0
 	for i := range a { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		b := a[:i+1] // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		x += b[0]    // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		b := a[:i+1] // ERROR "Proved IsSliceInBounds$"
+		x += b[0]    // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -43,7 +43,7 @@ func f1(a []int) int {
 func f2(a []int) int {
 	x := 0
 	for i := 1; i < len(a); i++ { // ERROR "Induction variable: limits \[1,\?\), increment 1$"
-		x += a[i] // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += a[i] // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -51,7 +51,7 @@ func f2(a []int) int {
 func f4(a [10]int) int {
 	x := 0
 	for i := 0; i < len(a); i += 2 { // ERROR "Induction variable: limits \[0,8\], increment 2$"
-		x += a[i] // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += a[i] // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -88,9 +88,10 @@ func f5_int8(a [10]int) int {
 	return x
 }
 
+//go:noinline
 func f6(a []int) {
 	for i := range a { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		b := a[0:i] // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		b := a[0:i] // ERROR "Proved IsSliceInBounds$"
 		f6(b)
 	}
 }
@@ -98,7 +99,7 @@ func f6(a []int) {
 func g0a(a string) int {
 	x := 0
 	for i := 0; i < len(a); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		x += int(a[i]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -106,7 +107,7 @@ func g0a(a string) int {
 func g0b(a string) int {
 	x := 0
 	for i := 0; len(a) > i; i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		x += int(a[i]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -114,7 +115,7 @@ func g0b(a string) int {
 func g0c(a string) int {
 	x := 0
 	for i := len(a); i > 0; i-- { // ERROR "Induction variable: limits \(0,\?\], increment 1$"
-		x += int(a[i-1]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i-1]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -122,7 +123,7 @@ func g0c(a string) int {
 func g0d(a string) int {
 	x := 0
 	for i := len(a); 0 < i; i-- { // ERROR "Induction variable: limits \(0,\?\], increment 1$"
-		x += int(a[i-1]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i-1]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -130,7 +131,7 @@ func g0d(a string) int {
 func g0e(a string) int {
 	x := 0
 	for i := len(a) - 1; i >= 0; i-- { // ERROR "Induction variable: limits \[0,\?\], increment 1$"
-		x += int(a[i]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -138,7 +139,7 @@ func g0e(a string) int {
 func g0f(a string) int {
 	x := 0
 	for i := len(a) - 1; 0 <= i; i-- { // ERROR "Induction variable: limits \[0,\?\], increment 1$"
-		x += int(a[i]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -147,7 +148,7 @@ func g1() int {
 	a := "evenlength"
 	x := 0
 	for i := 0; i < len(a); i += 2 { // ERROR "Induction variable: limits \[0,8\], increment 2$"
-		x += int(a[i]) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		x += int(a[i]) // ERROR "Proved IsInBounds$"
 	}
 	return x
 }
@@ -157,7 +158,7 @@ func g2() int {
 	x := 0
 	for i := 0; i < len(a); i += 2 { // ERROR "Induction variable: limits \[0,8\], increment 2$"
 		j := i
-		if a[i] == 'e' { // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		if a[i] == 'e' { // ERROR "Proved IsInBounds$"
 			j = j + 1
 		}
 		x += int(a[j])
@@ -168,29 +169,29 @@ func g2() int {
 func g3a() {
 	a := "this string has length 25"
 	for i := 0; i < len(a); i += 5 { // ERROR "Induction variable: limits \[0,20\], increment 5$"
-		useString(a[i:])   // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useString(a[:i+3]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useString(a[:i+5]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useString(a[i:])   // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed \(by limit\)$"
+		useString(a[:i+3]) // ERROR "Proved IsSliceInBounds$"
+		useString(a[:i+5]) // ERROR "Proved IsSliceInBounds$"
 		useString(a[:i+6])
 	}
 }
 
 func g3b(a string) {
 	for i := 0; i < len(a); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		useString(a[i+1:]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useString(a[i+1:]) // ERROR "Proved IsSliceInBounds$"
 	}
 }
 
 func g3c(a string) {
 	for i := 0; i < len(a); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		useString(a[:i+1]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useString(a[:i+1]) // ERROR "Proved IsSliceInBounds$"
 	}
 }
 
 func h1(a []byte) {
 	c := a[:128]
 	for i := range c { // ERROR "Induction variable: limits \[0,128\), increment 1$"
-		c[i] = byte(i) // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		c[i] = byte(i) // ERROR "Proved IsInBounds$"
 	}
 }
 
@@ -207,11 +208,11 @@ func k0(a [100]int) [100]int {
 			continue
 		}
 		a[i-11] = i
-		a[i-10] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
-		a[i-5] = i  // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
-		a[i] = i    // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
-		a[i+5] = i  // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
-		a[i+10] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		a[i-10] = i // ERROR "Proved IsInBounds$"
+		a[i-5] = i  // ERROR "Proved IsInBounds$"
+		a[i] = i    // ERROR "Proved IsInBounds$"
+		a[i+5] = i  // ERROR "Proved IsInBounds$"
+		a[i+10] = i // ERROR "Proved IsInBounds$"
 		a[i+11] = i
 	}
 	return a
@@ -224,12 +225,12 @@ func k1(a [100]int) [100]int {
 			continue
 		}
 		useSlice(a[:i-11])
-		useSlice(a[:i-10]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[:i-5])  // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[:i])    // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[:i+5])  // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[:i+10]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[:i+11]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useSlice(a[:i-10]) // ERROR "Proved IsSliceInBounds$"
+		useSlice(a[:i-5])  // ERROR "Proved IsSliceInBounds$"
+		useSlice(a[:i])    // ERROR "Proved IsSliceInBounds$"
+		useSlice(a[:i+5])  // ERROR "Proved IsSliceInBounds$"
+		useSlice(a[:i+10]) // ERROR "Proved IsSliceInBounds$"
+		useSlice(a[:i+11]) // ERROR "Proved IsSliceInBounds$"
 		useSlice(a[:i+12])
 
 	}
@@ -242,13 +243,13 @@ func k2(a [100]int) [100]int {
 			// This is a trick to prohibit sccp to optimize out the following out of bound check
 			continue
 		}
-		useSlice(a[i-11:])
-		useSlice(a[i-10:]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[i-5:])  // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[i:])    // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[i+5:])  // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[i+10:]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
-		useSlice(a[i+11:]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useSlice(a[i-11:]) // ERROR "Proved slicemask not needed \(by limit\)$"
+		useSlice(a[i-10:]) // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed \(by limit\)$"
+		useSlice(a[i-5:])  // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed \(by limit\)$"
+		useSlice(a[i:])    // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed \(by limit\)$"
+		useSlice(a[i+5:])  // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed \(by limit\)$"
+		useSlice(a[i+10:]) // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed \(by limit\)$"
+		useSlice(a[i+11:]) // ERROR "Proved IsSliceInBounds$"
 		useSlice(a[i+12:])
 	}
 	return a
@@ -261,7 +262,7 @@ func k3(a [100]int) [100]int {
 			continue
 		}
 		a[i+9] = i
-		a[i+10] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		a[i+10] = i // ERROR "Proved IsInBounds$"
 		a[i+11] = i
 	}
 	return a
@@ -274,7 +275,7 @@ func k3neg(a [100]int) [100]int {
 			continue
 		}
 		a[i+9] = i
-		a[i+10] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		a[i+10] = i // ERROR "Proved IsInBounds$"
 		a[i+11] = i
 	}
 	return a
@@ -287,7 +288,7 @@ func k3neg2(a [100]int) [100]int {
 			continue
 		}
 		a[i+9] = i
-		a[i+10] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		a[i+10] = i // ERROR "Proved IsInBounds$"
 		a[i+11] = i
 	}
 	return a
@@ -298,7 +299,7 @@ func k4(a [100]int) [100]int {
 	// and it isn't worth adding that special case to prove.
 	min := (-1)<<63 + 1
 	for i := min; i < min+50; i++ { // ERROR "Induction variable: limits \[-9223372036854775807,-9223372036854775757\), increment 1$"
-		a[i-min] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		a[i-min] = i // ERROR "Proved IsInBounds$"
 	}
 	return a
 }
@@ -306,8 +307,8 @@ func k4(a [100]int) [100]int {
 func k5(a [100]int) [100]int {
 	max := (1 << 63) - 1
 	for i := max - 50; i < max; i++ { // ERROR "Induction variable: limits \[9223372036854775757,9223372036854775807\), increment 1$"
-		a[i-max+50] = i   // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
-		a[i-(max-70)] = i // ERROR "(\([0-9]+\) )?Proved IsInBounds$"
+		a[i-max+50] = i   // ERROR "Proved IsInBounds$"
+		a[i-(max-70)] = i // ERROR "Proved IsInBounds$"
 	}
 	return a
 }
@@ -373,22 +374,22 @@ func d4() {
 }
 
 func d5() {
-	for i := int64(math.MinInt64 + 9); i > math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775803,-9223372036854775799\], increment 4"
+	for i := int64(math.MinInt64 + 9); i > math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775803,-9223372036854775799\], increment 4$"
 		useString("foo")
 	}
-	for i := int64(math.MinInt64 + 8); i > math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775804,-9223372036854775800\], increment 4"
+	for i := int64(math.MinInt64 + 8); i > math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775804,-9223372036854775800\], increment 4$"
 		useString("foo")
 	}
 	for i := int64(math.MinInt64 + 7); i > math.MinInt64+2; i -= 4 {
 		useString("foo")
 	}
-	for i := int64(math.MinInt64 + 6); i > math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775802,-9223372036854775802\], increment 4"
+	for i := int64(math.MinInt64 + 6); i > math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775802,-9223372036854775802\], increment 4$"
 		useString("foo")
 	}
-	for i := int64(math.MinInt64 + 9); i >= math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775803,-9223372036854775799\], increment 4"
+	for i := int64(math.MinInt64 + 9); i >= math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775803,-9223372036854775799\], increment 4$"
 		useString("foo")
 	}
-	for i := int64(math.MinInt64 + 8); i >= math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775804,-9223372036854775800\], increment 4"
+	for i := int64(math.MinInt64 + 8); i >= math.MinInt64+2; i -= 4 { // ERROR "Induction variable: limits \[-9223372036854775804,-9223372036854775800\], increment 4$"
 		useString("foo")
 	}
 	for i := int64(math.MinInt64 + 7); i >= math.MinInt64+2; i -= 4 {
@@ -409,23 +410,23 @@ func bce1() {
 		panic("invalid test: modulos should differ")
 	}
 
-	for i := b; i < a; i += z { // ERROR "Induction variable: limits \[-1547,9223372036854772720\], increment 1337"
+	for i := b; i < a; i += z { // ERROR "Induction variable: limits \[-1547,9223372036854772720\], increment 1337$"
 		useString("foobar")
 	}
 }
 
 func nobce2(a string) {
 	for i := int64(0); i < int64(len(a)); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		useString(a[i:]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useString(a[i:]) // ERROR "Proved IsSliceInBounds$"
 	}
 	for i := int64(0); i < int64(len(a))-31337; i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
-		useString(a[i:]) // ERROR "(\([0-9]+\) )?Proved IsSliceInBounds$"
+		useString(a[i:]) // ERROR "Proved IsSliceInBounds$" "Proved slicemask not needed"
 	}
-	for i := int64(0); i < int64(len(a))+int64(-1<<63); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$" "Disproved Less64"
+	for i := int64(0); i < int64(len(a))+int64(-1<<63); i++ { // ERROR "Disproved Less64$" "Induction variable: limits \[0,\?\), increment 1$"
 		useString(a[i:])
 	}
 	j := int64(len(a)) - 123
-	for i := int64(0); i < j+123+int64(-1<<63); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$" "Disproved Less64"
+	for i := int64(0); i < j+123+int64(-1<<63); i++ { // ERROR "Disproved Less64$" "Induction variable: limits \[0,\?\), increment 1$"
 		useString(a[i:])
 	}
 	for i := int64(0); i < j+122+int64(-1<<63); i++ { // ERROR "Induction variable: limits \[0,\?\), increment 1$"
@@ -454,16 +455,16 @@ func issue26116a(a []int) {
 
 func stride1(x *[7]int) int {
 	s := 0
-	for i := 0; i <= 8; i += 3 { // ERROR "Induction variable: limits \[0,6\], increment 3"
-		s += x[i] // ERROR "Proved IsInBounds"
+	for i := 0; i <= 8; i += 3 { // ERROR "Induction variable: limits \[0,6\], increment 3$"
+		s += x[i] // ERROR "Proved IsInBounds$"
 	}
 	return s
 }
 
 func stride2(x *[7]int) int {
 	s := 0
-	for i := 0; i < 9; i += 3 { // ERROR "Induction variable: limits \[0,6\], increment 3"
-		s += x[i] // ERROR "Proved IsInBounds"
+	for i := 0; i < 9; i += 3 { // ERROR "Induction variable: limits \[0,6\], increment 3$"
+		s += x[i] // ERROR "Proved IsInBounds$"
 	}
 	return s
 }
